/*
	SHARP PC-G800 series emulator
	ˑ
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "g800.h"

#include "SDL.h"
#include "symbol.h"
#include "keytop.h"
#include "g800icon.xpm"

#define MAX_ZOOM	3
#define MAX_SCALE	(360 / 8 + 1)
#define SCREEN(x, y) ((Uint8 *)screen->pixels + (y) * screen->pitch + screen->format->BytesPerPixel * (x))
#define VRAM_OFF(col, row, top)	(((row + (top) / 8) % 8) * vramWidth + (col) * cellWidth)
#define VRAM(col, row, top) &vram[VRAM_OFF(col, row, top)]
#define OLD_VRAM(col, row, top) &oldVram[VRAM_OFF(col, row, top)]

#define COLOR_BODY	0
#define COLOR_GRAY	1
#define COLOR_GREEN	2
#define COLOR_YELLOW	3
#define COLOR_RED	4
#define COLOR_LIGHTGRAY	5
#define COLOR_LIGHTGREEN	6
#define COLOR_LIGHTYELLOW	7
#define COLOR_LIGHTRED	8
#define COLOR_DARKGRAY	9
#define COLOR_DARKGREEN	10
#define COLOR_DARKYELLOW	11
#define COLOR_DARKRED	12
#define COLOR_LCD_START	16

#define KEYDOWN_MSEC	200

const static Bitmap *bmpStatus[] = {
	bmpStatus1,
	bmpStatus2,
	bmpStatus3
};
const static Bitmap *bmpKeytop[] = {
	bmpKeytop1,
	bmpKeytop2,
	bmpKeytop3
};
const static uint8 keyBackColor[] = {
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GREEN, COLOR_GREEN, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_GRAY, COLOR_YELLOW, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY, COLOR_GRAY,
	COLOR_RED, COLOR_GRAY
};
const static uint8 keyForeColor[] = {
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGREEN, COLOR_LIGHTGREEN, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTGRAY, COLOR_LIGHTYELLOW, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY, COLOR_LIGHTGRAY,
	COLOR_LIGHTRED, COLOR_LIGHTGRAY
};

const static SDL_VideoInfo *videoInfo;
static SDL_Joystick *joy;
static SDL_AudioSpec audio;

static SDL_Rect rectLCD;
static SDL_Rect rectLCDmain;
static SDL_Rect rectLCDstatus[STATUS_LAST + 1];
static int statusRow[STATUS_LAST + 1];
static uint8 statusMask[STATUS_LAST + 1];
static int zoomedBpp;
static int zoomedPitch;
static SDL_Color colorTable[256];
static Uint32 pixelTable[256];

static SDL_Surface *screen;
static Uint8 *pixmapBack = NULL;
static Uint8 *pixmapOff;
static Uint8 *pixmapOn;
static Uint8 *pixmapStatusOff[STATUS_LAST + 1];
static Uint8 *pixmapStatusOn[STATUS_LAST + 1];
static Uint8 *pixmapTable[MAX_SCALE];
static Uint8 *pixmapStatusTable[MAX_SCALE][STATUS_LAST + 1];
static Uint8 *pixmapButton;

static uint8 oldVram[166 * 9];
static uint8 oldLcdTop;

static SDL_Rect rectKey[GKEY_DOUBLE];
static uint16 swKey = 0;
static int countNumlock;
static int countCapslock;
static int countScrollock;

/*
	LCD̔Zx̍ől𓾂
*/
static inline int getScaleMax(void)
{
	if(lcdScales == 2)
		return 1;
	else
		return freqUpdateIO / 8;
}

/*
	CLCDɓ_`
*/
static inline void putDot(Uint8 *dst, const void *pix)
{
	Uint8 *p = dst;

	for(p = dst; p != dst + zoomedPitch; p += screen->pitch)
		memcpy(p, pix, zoomedBpp);
}

/*
	Xe[^X`
*/
static inline void putPattern(Uint8 *dst, const Uint8 *img, int w, int h)
{
	Uint8 *p;
	const Uint8 *q;

	for(
	p = dst, q = img;
	p != dst + screen->pitch * h;
	p += screen->pitch, q += w * screen->format->BytesPerPixel
	)
		memcpy(p, q, w * screen->format->BytesPerPixel);
}

/*
	Xe[^X\(updateLCD1̉)
*/
static inline void updateStatus(SDL_Rect **rect, int status, int chk)
{
	int row = statusRow[status], shift = lcdTop % 8;
	uint8 mask = statusMask[status], pat, *old_pat;
	SDL_Rect *r;

	pat     = *VRAM(lcdCols, row, lcdTop) >> shift | *VRAM(lcdCols, row + 1, lcdTop) << (8 - shift);
	old_pat = OLD_VRAM(lcdCols, row, lcdTop);
	if(!((pat ^ *old_pat) & mask) && chk)
		return;

	r = &rectLCDstatus[status];
	if(pat & mask) {
		putPattern(SCREEN(r->x, r->y), pixmapStatusOn[status], r->w, r->h);
		*old_pat |= mask;
	} else {
		putPattern(SCREEN(r->x, r->y), pixmapStatusOff[status], r->w, r->h);
		*old_pat &= ~mask;
	}
	*(*rect)++ = *r;
}

/*
	cell\(updateLCD1̉)
*/
static inline void updateCell(SDL_Rect **rect, int col, int row, int chk)
{
	int begin, d, end;
	uint8 *p_vram, *p_oldvram, pat, changed, mask;
	Uint8 *p_screen, *p_screen0, *p_screen00;

	p_vram    = VRAM(col, row, lcdTop);
	p_oldvram = OLD_VRAM(col, row, 0);
	if(chk) {
		if(memcmp(p_vram, p_oldvram, 6) == 0)
			return;
	} else {
		mask = 0xff >> (8 - cellHeight);
		*(p_oldvram + 0) = ~*(p_vram + 0) & mask;
		*(p_oldvram + 1) = ~*(p_vram + 1) & mask;
		*(p_oldvram + 2) = ~*(p_vram + 2) & mask;
		*(p_oldvram + 3) = ~*(p_vram + 3) & mask;
		*(p_oldvram + 4) = ~*(p_vram + 4) & mask;
		if(cellWidth == 6)
			*(p_oldvram + 5) = ~*(p_vram + 5) & mask;
	}

	if((d = lcdTop % 8) == 0)
		end = begin = 0;
	else if(row == 0) {
		begin = d;
		end = 0;
	} else if(row == lcdRows) {
		begin = 0;
		end = cellHeight - d;
	} else
		end = begin = 0;

	(*rect)->x = zoom * col * 6 + rectLCDmain.x;
	(*rect)->y = zoom * (row * 8 + begin - d) + rectLCDmain.y;
	(*rect)->w = zoom * cellWidth;
	(*rect)->h = zoom * (cellHeight - end - begin);
	p_screen00 = SCREEN((*rect)->x, (*rect)->y);

	for(
	p_screen0 = p_screen00;
	p_screen0 != p_screen00 + (*rect)->w * screen->format->BytesPerPixel;
	p_screen0 += zoomedBpp
	) {
		if(*p_vram != *p_oldvram) {
			for(
			changed = ((*p_vram >> begin) ^ (*p_oldvram >> begin)) & (0xff >> end), pat = *p_vram >> begin, p_screen = p_screen0;
			changed != 0;
			changed >>= 1, pat >>= 1, p_screen += zoomedPitch
			)
				if(changed & 1)
					putDot(p_screen, (pat & 1 ? pixmapOn: pixmapOff));
			*p_oldvram = *p_vram;
		}
		p_vram++;
		p_oldvram++;
	}

	if((*rect)->y == (*rect - 1)->y && (*rect)->x == (*rect - 1)->x + (*rect - 1)->w)
		(*rect - 1)->w += (*rect)->w;
	else
		(*rect)++;
}

/*
	LCDXV(cȂ)
*/
static int updateLCD1(void)
{
	static SDL_Rect rect[255] = { { 0, 0, 0, 0 } };
	SDL_Rect *p_rect = &rect[1];
	int chk, col, row, status, must_lock;

	if(lcdTop == oldLcdTop)
		chk = TRUE;
	else {
		chk = FALSE;
		oldLcdTop = lcdTop;
	}

	if((must_lock = SDL_MUSTLOCK(screen)))
		if(SDL_LockSurface(screen) < 0)
			return FALSE;
	for(status = STATUS_FIRST; status != STATUS_LAST + 1; status++)
		updateStatus(&p_rect, status, chk);
	for(row = 0; row != lcdRows + (lcdTop % 8 != 0); row++)
		for(col = 0; col != lcdCols; col++)
			updateCell(&p_rect, col, row, chk);
	if(must_lock)
		SDL_UnlockSurface(screen);

	if(p_rect == &rect[1])
		return FALSE;
	SDL_UpdateRects(screen, (int )(p_rect - rect - 1), &rect[1]);

	return TRUE;
}

/*
	LCDXV(c)
*/
static int updateLCD2(void)
{
	static struct {
		uint8 oldpat[(144 + 1) * 8];
	} page[MAX_SCALE], *p_page = &page[1];
	static int lcd_scale[48 * (144 + 1)];
	static SDL_Rect rect[144 * 8] = { { 0, 0, 0, 0 } };
	SDL_Rect *p_rect = &rect[1];
	int x, x0, y, status, shift, *p_scale;
	Uint8 *p_screen00, *p_screen0, *p_screen, *p;
	uint8 pat, bit, *p_vram0, *p_vram1, *p_oldpat;

	if(SDL_MUSTLOCK(screen))
		if(SDL_LockSurface(screen) < 0)
			return FALSE;

	/* Xe[^X` */
	shift = lcdTop % 8;
	p_scale = lcd_scale;
	for(status = STATUS_FIRST; status != STATUS_LAST + 1; status++) {
		p_vram0 = &vram[((statusRow[status] + lcdTop / 8 + 0) % 8) * vramWidth + vramWidth - 1];
		p_vram1 = &vram[((statusRow[status] + lcdTop / 8 + 1) % 8) * vramWidth + vramWidth - 1];
		pat = *p_vram0 >> shift | *p_vram1 << (8 - shift);
		p_oldpat = &p_page->oldpat[statusRow[status] * vramWidth + vramWidth - 1];
		if(!((pat ^ *p_oldpat) & statusMask[status])) {
			p_scale++;
			continue;
		}

		if(*p_oldpat & statusMask[status])
			(*p_scale)--;
		if(pat & statusMask[status])
			(*p_scale)++;

		*p_rect = rectLCDstatus[status];
		putPattern(SCREEN(p_rect->x, p_rect->y), pixmapStatusTable[*p_scale++][status], p_rect->w, p_rect->h);
		p_rect++;
		
		if(pat & statusMask[status])
			*p_oldpat |= statusMask[status];
		else
			*p_oldpat &= ~statusMask[status];
	}

	/* CLCD` */
	p_vram0 = &vram[((lcdTop / 8) + 0) * vramWidth];
	p_vram1 = p_vram0 + vramWidth;
	p_oldpat = p_page->oldpat;
	for(
	y = 0, p_screen00 = SCREEN(rectLCDmain.x, rectLCDmain.y);
	y != lcdHeight;
	y += 8, p_screen00 += zoomedPitch * 8
	) {
		if(p_vram0 == &vram[vramRows * vramWidth])
			p_vram0 = vram;
		if(p_vram1 == &vram[vramRows * vramWidth])
			p_vram1 = vram;

		for(
		x0 = 0, p_screen0 = p_screen00;
		x0 != 144;
		x0 += 6, p_screen0 += zoomedBpp * 6
		) {
			for(
			x = x0, p_screen = p_screen0;
			x != x0 + cellWidth;
			x++, p_screen += zoomedBpp, p_vram0++, p_vram1++, p_oldpat++
			) {
				pat = *p_vram0 >> shift | *p_vram1 << (8 - shift);
				if(pat == *p_oldpat) {
					p_scale += cellHeight;
					continue;
				}

				for(
				p = p_screen, bit = 1;
				p != p_screen + cellHeight * zoomedPitch;
				p += zoomedPitch, bit <<= 1
				) {
					if(*p_oldpat & bit)
						(*p_scale)--;
					if(pat & bit)
						(*p_scale)++;
					
					putDot(p, pixmapTable[*p_scale]);
					p_scale++;
				}
				*p_oldpat = pat;

				p_rect->x = x * zoom + rectLCDmain.x;
				p_rect->y = y * zoom + rectLCDmain.y;
				p_rect->w = 1 * zoom;
				p_rect->h = cellHeight * zoom;
				if(p_rect->y == (p_rect - 1)->y && p_rect->x == (p_rect - 1)->x + (p_rect - 1)->w)
					(p_rect - 1)->w += p_rect->w;
				else
					p_rect++;
			}
		}

		p_vram0++;
		p_vram1++;
		p_oldpat++;
	}
	p_page = (p_page != &page[getScaleMax() - 1] ? p_page + 1: page);

	if(SDL_MUSTLOCK(screen))
		SDL_UnlockSurface(screen);
	SDL_UpdateRects(screen, (int )(p_rect - &rect[1]), &rect[1]);

	return TRUE;
}

/*
	ԕ\XV
*/
static void updateCaption(void)
{
	static char old_caption[512] = "*";
	char caption[512], buf[512];

	switch(sioMode) {
	case SIO_MODE_STOP:
		strcpy(caption, "");
		break;
	case SIO_MODE_IN:
		sprintf(caption, " - IN:%d/%dbytes %s", sioCount / 14, sioBufferSize, pathSioIn);
		break;
	case SIO_MODE_OUT:
		sprintf(caption, " - OUT:%dbytes %s", sioCount / 10, pathSioOut);
		break;
	}

	if(strcmp(caption, old_caption) == 0)
		return;
	strcpy(old_caption, caption);

	sprintf(buf, "POCKET COMPUTER %s%s", machineInfo[machine]->machine_name, caption);
	SDL_WM_SetCaption(buf, NULL);
}

/*
	LCDXV
*/
int updateLCD(void)
{
	updateCaption();
	if(lcdScales == 2)
		return updateLCD1();
	else
		return updateLCD2();
}

/*
	LCDNA(updateLCDContrast̉)
*/
static void clearLCD(void)
{
	Uint8 *p;

	if(SDL_MUSTLOCK(screen))
		if(SDL_LockSurface(screen) < 0)
			return;

	for(
	p = screen->pixels + (screen->pitch * rectLCD.y + screen->format->BytesPerPixel * rectLCD.x);
	p != screen->pixels + (screen->pitch * (rectLCD.y + rectLCD.h) + screen->format->BytesPerPixel * rectLCD.x);
	p += screen->pitch
	)
		memcpy(p, pixmapBack, rectLCD.w * screen->format->BytesPerPixel);
	if(SDL_MUSTLOCK(screen))
		SDL_UnlockSurface(screen);
	SDL_UpdateRect(screen, 0, 0, screen->w, screen->h);
	oldLcdTop = 0xff;
}

/*
	hԂ(updateLCDContrast̉)
*/
static Uint8 *fillPixmap(Uint8 *pixmap, const void *pix, int size)
{
	Uint8 *p;

	for(p = pixmap; p < pixmap + size; p += screen->format->BytesPerPixel)
		memcpy(p, pix, screen->format->BytesPerPixel);
	return pixmap;
}

/*
	BitmapPixmapɕϊ(updateLCDContrast̉)
*/
static Uint8 *makePixmap(Uint8 *pixmap, SDL_Rect rect, Bitmap bitmap, const void *pix0, const void *pix1)
{
	int i, x, y;
	uint8 mask, *r = bitmap.image;
	Uint8 *p, *q;

	if(rect.w <= 0)
		return NULL;

	fillPixmap(pixmap, pix0, rect.w * rect.h * screen->format->BytesPerPixel);
	for(
	y = 0, p = pixmap;
	y != bitmap.height && y != rect.h;
	y++, p += rect.w * screen->format->BytesPerPixel
	) {
		q = p;
		for(
		x = 0;
		x < bitmap.width;
		x += 8
		) {
			for(
			i = x, mask = 0x80;
			i < x + 8 && i < bitmap.width && i < rect.w;
			i++, mask >>= 1
			) {
				if(*r & mask)
					memcpy(q, pix1, screen->format->BytesPerPixel);
				q += screen->format->BytesPerPixel;
			}
			r++;
		}
	}

	return pixmap;
}

/*
	LCDd߂(UpdateLCDContrast̉)
*/
static int getLcdVoltage(void)
{
	int voltage;

	if((lcdEffectBlack && lcdEffectWhite) || lcdDisabled)
		return 0;
	else if(lcdEffectBlack)
		return 0x20;
	else if(lcdEffectWhite)
		return 0;
	else {
		if((voltage = lcdContrast + (lcdEffectDark ? 2: 0)) > 0x20)
			return 0x20;
		else
			return voltage;
	}
}

/*
	LCD̐F߂(UpdateLCDContrast̉)
*/
static SDL_Color *getLcdColors(SDL_Color *colors, int voltage)
{
	SDL_Color back = { 0 }, on = { 0 }, off = { 0 };
	int c1, c2;

	/* F𓾂 */
	back.r = (colorBack >> 16) & 0xff;
	back.g = (colorBack >>  8) & 0xff;
	back.b = (colorBack >>  0) & 0xff;
	if(!lcdEffectReverse) {
		off.r  = (colorOff  >> 16) & 0xff;
		off.g  = (colorOff  >>  8) & 0xff;
		off.b  = (colorOff  >>  0) & 0xff;
		on.r   = (colorOn   >> 16) & 0xff;
		on.g   = (colorOn   >>  8) & 0xff;
		on.b   = (colorOn   >>  0) & 0xff;
	} else {
		off.r  = (colorOn  >> 16) & 0xff;
		off.g  = (colorOn  >>  8) & 0xff;
		off.b  = (colorOn  >>  0) & 0xff;
		on.r   = (colorOff >> 16) & 0xff;
		on.g   = (colorOff >>  8) & 0xff;
		on.b   = (colorOff >>  0) & 0xff;
	}
	if(voltage <= 0x0f) {
		c1 = 0x0f - voltage;
		c2 = voltage;
		colors[0]   = back;
		colors[1].r = (back.r * c1 + off.r * c2) / 0x0f;
		colors[1].g = (back.g * c1 + off.g * c2) / 0x0f;
		colors[1].b = (back.b * c1 + off.b * c2) / 0x0f;
		colors[2].r = (back.r * c1 + on.r  * c2) / 0x0f;
		colors[2].g = (back.g * c1 + on.g  * c2) / 0x0f;
		colors[2].b = (back.b * c1 + on.b  * c2) / 0x0f;
	} else {
		c1 = 0x20 - voltage;
		c2 = voltage - 0x0f;
		colors[0].r = (back.r * c1 + off.r * c2) / 0x11;
		colors[0].g = (back.g * c1 + off.g * c2) / 0x11;
		colors[0].b = (back.b * c1 + off.b * c2) / 0x11;
		colors[1].r = (off.r  * c1 + on.r  * c2) / 0x11;
		colors[1].g = (off.g  * c1 + on.g  * c2) / 0x11;
		colors[1].b = (off.b  * c1 + on.b  * c2) / 0x11;
		colors[2]   = on;
	}

	return colors;
}

/*
	LCD̐F(c)߂(UpdateLCDContrast̉)
*/
static SDL_Color getLcdColorScale(SDL_Color off, SDL_Color on, int max, int i)
{
	SDL_Color result = { 0 };

	if(lcdScales != 0)
		i = (i * lcdScales / (max + 1)) * max / (lcdScales - 1);
	result.r = (off.r * (max - i) + on.r * i) / max;
	result.g = (off.g * (max - i) + on.g * i) / max;
	result.b = (off.b * (max - i) + on.b * i) / max;
	return result;
}

/*
	RgXgXV
*/
void updateLCDContrast(void)
{
	const SDL_Color color_black       = { 0x00, 0x00, 0x00 };
	const SDL_Color color_gray        = { 0x50, 0x50, 0x70 };
	const SDL_Color color_green       = { 0x20, 0x70, 0x60 };
	const SDL_Color color_yellow      = { 0xb0, 0x90, 0x30 };
	const SDL_Color color_red         = { 0xb0, 0x50, 0x50 };
	const SDL_Color color_lightgray   = { 0xcc, 0xcc, 0xee };
	const SDL_Color color_lightgreen  = { 0xdd, 0xff, 0xcc };
	const SDL_Color color_lightyellow = { 0xff, 0xee, 0xdd };
	const SDL_Color color_lightred    = { 0xff, 0xcc, 0xcc };
	const SDL_Color color_white       = { 0xff, 0xff, 0xff };
	SDL_Color base[3];
	int i, cont, z, status, voltage = getLcdVoltage(), max = getScaleMax();

	/* F */
	colorTable[COLOR_BODY]   = color_black;
	colorTable[COLOR_GRAY]   = color_gray;
	colorTable[COLOR_GREEN]  = color_green;
	colorTable[COLOR_YELLOW] = color_yellow;
	colorTable[COLOR_RED]    = color_red;
	if(zoom > 1)
		colorTable[COLOR_LIGHTGRAY] = colorTable[COLOR_LIGHTGREEN] = colorTable[COLOR_LIGHTYELLOW] = colorTable[COLOR_LIGHTRED] = color_white;
	else {
		colorTable[COLOR_LIGHTGRAY]   = color_lightgray;
		colorTable[COLOR_LIGHTGREEN]  = color_lightgreen;
		colorTable[COLOR_LIGHTYELLOW] = color_lightyellow;
		colorTable[COLOR_LIGHTRED]    = color_lightred;
	}

	/* LCD̐F */
	getLcdColors(base, voltage);
	colorTable[COLOR_LCD_START + 0] = base[0];
	for(cont = 0, i = COLOR_LCD_START + 1; cont <= max; cont++, i++)
		colorTable[i] = getLcdColorScale(base[1], base[2], max, cont);

	/* Pixel */
	if(screen->format->BytesPerPixel == 1) {
		for(i = 0; i < 256; i++)
			*(Uint8 *)&pixelTable[i] = i;
		SDL_SetPalette(screen, SDL_PHYSPAL, colorTable, 0, COLOR_LCD_START + max + 2);
	} else {
		for(i = 0; i <= COLOR_LCD_START + max + 1; i++)
			pixelTable[i] = SDL_MapRGB(screen->format, colorTable[i].r, colorTable[i].g, colorTable[i].b);
	}

	/* 摜(Rs[) */
	fillPixmap(pixmapBack, &pixelTable[COLOR_LCD_START + 0], zoomedPitch);
	for(cont = 0, i = COLOR_LCD_START + 1; cont <= max; cont++, i++)
		fillPixmap(pixmapTable[cont], &pixelTable[i], zoomedBpp);
	z = (zoom < MAX_ZOOM ? zoom: MAX_ZOOM) - 1;
	for(status = STATUS_FIRST; status <= STATUS_LAST; status++) {
		makePixmap(pixmapStatusOff[status], rectLCDstatus[status], bmpStatus[z][status], &pixelTable[COLOR_LCD_START + 0], &pixelTable[COLOR_LCD_START + 1]);
		for(cont = 0, i = COLOR_LCD_START + 1; cont <= max; cont++, i++)
			makePixmap(pixmapStatusTable[cont][status], rectLCDstatus[status], bmpStatus[z][status], &pixelTable[COLOR_LCD_START], &pixelTable[i]);
	}

	/* ʂĕ\ */
	clearLCD();
	updateLCD1();
}

/*
	ʒu߂(updateLayout̉)
*/
static SDL_Rect setLayout(Rect r)
{
	SDL_Rect result;

	result.x = r.x * zoom / PIXEL_WIDTH;
	result.y = r.y * zoom / PIXEL_HEIGHT;
	result.w = r.w * zoom / PIXEL_WIDTH;
	result.h = r.h * zoom / PIXEL_HEIGHT;
	return result;
}

/*
	CAEgXV
*/
void updateLayout(void)
{
	const Machineinfo *m;
	SDL_Rect rect_function_key;
	int i, gkey, size, status, z = (zoom < MAX_ZOOM ? zoom: MAX_ZOOM) - 1, first = (pixmapBack == NULL);
	char buf[256];

	/* }VŗL̏s */
	m = machineInfo[machine];
	sprintf(buf, "POCKET COMPUTER %s", m->machine_name);
	SDL_WM_SetCaption(buf, NULL);
	memcpy(statusRow,  m->status_row,  sizeof(statusRow));
	memcpy(statusMask, m->status_mask, sizeof(statusMask));

	/* CAEgݒ肷 */
	rectLCD     = setLayout(m->pos_lcd);
	rectLCDmain = setLayout(m->pos_lcd_main);
	for(i = 0; i < sizeof(rectKey) / sizeof(rectKey[0]); i++)
		rectKey[i] = setLayout(m->pos_key[i]);
	for(status = STATUS_FIRST; status <= STATUS_LAST; status++)
		rectLCDstatus[status] = setLayout(m->pos_lcd_status[status]);
	rect_function_key = setLayout(m->pos_function_key);

	/* EBhE\ */
	if((screen = SDL_SetVideoMode(rectLCD.x + rectLCD.w + (useSoftwareKey ? rect_function_key.w: 0), rectLCD.y + rectLCD.h, videoInfo->vfmt->BitsPerPixel, SDL_HWSURFACE | (videoInfo->vfmt->BitsPerPixel <= 8 ? SDL_HWPALETTE: 0))) == NULL) {
		fprintf(stderr, "SDL_SetVideoMode fail. %s\n", SDL_GetError());
		exit(1);
	}

	/* mۂ */
	zoomedBpp   = screen->format->BytesPerPixel * zoom;
	zoomedPitch = screen->pitch * zoom;
	pixmapBack = (first ? malloc(zoomedPitch): realloc(pixmapBack, zoomedPitch));
	for(i = 0; i < sizeof(pixmapTable) / sizeof(pixmapTable[0]); i++)
		pixmapTable[i] = (first ? malloc(zoomedBpp): realloc(pixmapTable[i], zoomedBpp));
	pixmapOff = pixmapTable[0];
	pixmapOn  = pixmapTable[getScaleMax()];
	for(status = STATUS_FIRST; status <= STATUS_LAST; status++) {
		size = rectLCDstatus[status].w * rectLCDstatus[status].h * screen->format->BytesPerPixel;
		pixmapStatusOff[status] = (first ? malloc(size): realloc(pixmapStatusOff[status], size));
		for(i = 0; i < sizeof(pixmapStatusTable) / sizeof(pixmapStatusTable[0]); i++)
			pixmapStatusTable[i][status] = (first ? malloc(size): realloc(pixmapStatusTable[i][status],  size));
		pixmapStatusOff[status] = pixmapStatusTable[0][status];
		pixmapStatusOn[status]  = pixmapStatusTable[getScaleMax()][status];
	}

	/* RgXgXV */
	updateLCDContrast();
	
	/* {^\ */
	if(!useSoftwareKey)
		return;
	size = rectKey[GKEY_OFF].w * rectKey[GKEY_OFF].h * screen->format->BytesPerPixel;
	pixmapButton = (first ? malloc(size): realloc(pixmapButton, size));
	for(gkey = GKEY_NONE; gkey <= GKEY_BREAK; gkey++) {
		if(rectKey[gkey].w <= 0)
			continue;
		makePixmap(pixmapButton, rectKey[gkey], bmpKeytop[z][gkey], &pixelTable[keyBackColor[gkey]], &pixelTable[keyForeColor[gkey]]);
		putPattern(SCREEN(rectKey[gkey].x, rectKey[gkey].y), pixmapButton, rectKey[gkey].w, rectKey[gkey].h);
	}
}

/*
	L[(updateKeỷ)
*/
static inline uint8 pressKey(uint16 k)
{
	switch(k) {
	case GKEY_NONE:
		return 0;
	case GKEY_BREAK:
		keyBreak |= 0x80;
		return INTERRUPT_KON;
	default:
		k = (k & 0xff) - 1;
		keyMatrix[k / 8] |= (1 << (k % 8));
		return INTERRUPT_IA;
	case GKEY_SHIFT:
		keyShift |= 0x01;
		return INTERRUPT_IA;
	case GKEY_RESET:
		keyReset = TRUE;
		return 0;
	case GKEY_MENU:
		return 0;
	case GKEY_DEBUG:
		return 0;
	case GKEY_11PIN1:
		pin11In |= 0x01; updateSerial();
		return 0;
	case GKEY_11PIN2:
		pin11In |= 0x02; updateSerial();
		return 0;
	case GKEY_11PIN3:
		pin11In |= 0x04; updateSerial();
		return 0;
	case GKEY_11PIN4:
		pin11In |= 0x08; updateSerial();
		return 0;
	case GKEY_11PIN5:
		pin11In |= 0x10; updateSerial();
		return 0;
	case GKEY_11PIN6:
		pin11In |= 0x20; updateSerial();
		return 0;
	case GKEY_11PIN7:
		pin11In |= 0x40; updateSerial();
		return 0;
	case GKEY_11PIN8:
		pin11In |= 0x80; updateSerial();
		return 0;
	}
}

/*
	L[𗣂(updateKeỷ)
*/
static inline void releaseKey(uint16 k)
{
	switch(k) {
	case GKEY_NONE:
		break;
	case GKEY_BREAK:
		keyBreak &= ~0x80;
		break;
	default:
		k = (k & 0xff) - 1;
		keyMatrix[k / 8] &= ~(1 << (k % 8));
		break;
	case GKEY_SHIFT:
		keyShift &= ~0x01;
		break;
	case GKEY_RESET:
		keyReset = FALSE;
		break;
	case GKEY_MENU:
		if(emulatorMode != EMULATOR_MODE_MENU) {
			emulatorMode = EMULATOR_MODE_MENU;
			menu();
			emulatorMode = EMULATOR_MODE_RUN;
		}
		break;
	case GKEY_DEBUG:
		z80.i.trace = !z80.i.trace;
		break;
	case GKEY_11PIN1:
		pin11In &= ~0x01; updateSerial();
		break;
	case GKEY_11PIN2:
		pin11In &= ~0x02; updateSerial();
		break;
	case GKEY_11PIN3:
		pin11In &= ~0x04; updateSerial();
		break;
	case GKEY_11PIN4:
		pin11In &= ~0x08; updateSerial();
		break;
	case GKEY_11PIN5:
		pin11In &= ~0x10; updateSerial();
		break;
	case GKEY_11PIN6:
		pin11In &= ~0x20; updateSerial();
		break;
	case GKEY_11PIN7:
		pin11In &= ~0x40; updateSerial();
		break;
	case GKEY_11PIN8:
		pin11In &= ~0x80; updateSerial();
		break;
	}
}

/*
	L[ԂXV
*/
uint8 updateKey(void)
{
	SDL_Event e;
	const SDL_Rect *rect_key;
	uint8 itype = 0;
	uint16 key;

	if(countNumlock > 0) {
		if(--countNumlock == 0)
			releaseKey(keyconv[SDLK_NUMLOCK]);
	} else if(countCapslock > 0) {
		if(--countCapslock == 0)
			releaseKey(keyconv[SDLK_CAPSLOCK]);
	} else if(countScrollock > 0) {
		if(--countScrollock == 0)
			releaseKey(keyconv[SDLK_SCROLLOCK]);
	}

	while(SDL_PollEvent(&e)) {
		switch(e.type) {
		case SDL_KEYDOWN:
			if(e.key.keysym.sym == 0)
				continue;
			else if(e.key.keysym.sym == SDLK_NUMLOCK)
				countNumlock = freqUpdateIO / 10;
			else if(e.key.keysym.sym == SDLK_CAPSLOCK)
				countCapslock = freqUpdateIO / 10;
			else if(e.key.keysym.sym == SDLK_SCROLLOCK)
				countScrollock = freqUpdateIO / 10;
			itype |= pressKey(keyconv[e.key.keysym.sym]);
			break;
		case SDL_KEYUP:
			if(e.key.keysym.sym == 0)
				continue;
			key = keyconv[e.key.keysym.sym];
			if(e.key.keysym.sym == SDLK_NUMLOCK) {
				countNumlock = freqUpdateIO / 10;
				itype |= pressKey(key);
			} else if(e.key.keysym.sym == SDLK_CAPSLOCK) {
				countCapslock = freqUpdateIO / 10;
				itype |= pressKey(key);
			} else if(e.key.keysym.sym == SDLK_SCROLLOCK) {
				countScrollock = freqUpdateIO / 10;
				itype |= pressKey(key);
			} else
				releaseKey(key);
			break;
		case SDL_JOYAXISMOTION:
			if(e.jaxis.axis == 0) {
				if(e.jaxis.value < -32768 / 10)
					itype |= pressKey(joyLeft);
				else
					releaseKey(joyLeft);
				if(e.jaxis.value > 32767 / 10)
					itype |= pressKey(joyRight);
				else
					releaseKey(joyRight);
			} else if(e.jaxis.axis == 1) {
				if(e.jaxis.value < -32768 / 10)
					itype |= pressKey(joyUp);
				else
					releaseKey(joyUp);
				if(e.jaxis.value > 32767 / 10)
					itype |= pressKey(joyDown);
				else
					releaseKey(joyDown);
			}
			break;
		case SDL_JOYBUTTONDOWN:
			if(e.jbutton.button >= JOY_BUTTONS && joyButton[e.jbutton.button] == 0xff)
				break;
			itype |= pressKey(joyButton[e.jbutton.button]);
			break;
		case SDL_JOYBUTTONUP:
			if(e.jbutton.button >= JOY_BUTTONS && joyButton[e.jbutton.button] == 0xff)
				break;
			releaseKey(joyButton[e.jbutton.button]);
			break;
		case SDL_MOUSEBUTTONDOWN:
			if(!useSoftwareKey)
				break;
			for(rect_key = &rectKey[GKEY_OFF]; rect_key <= &rectKey[GKEY_BREAK]; rect_key++)
				if(rect_key->x <= e.button.x && e.button.x < rect_key->x + rect_key->w && rect_key->y <= e.button.y && e.button.y < rect_key->y + rect_key->h) {
					swKey = (uint16 )(rect_key - rectKey);
					itype |= pressKey(swKey);
				}
			break;
		case SDL_MOUSEBUTTONUP:
			if(!useSoftwareKey)
				break;
			if(swKey == GKEY_NONE)
				break;
			releaseKey(swKey);
			swKey = GKEY_NONE;
			break;
		case SDL_QUIT:
			if(!closeAsOff)
				exit(0);
			pressKey(GKEY_OFF);
			break;
		}
	}

	return itype;
}

/*
	҂
*/
int delay(int interval)
{
	static Uint32 last = 0, left;
	Uint32 now;

	now = SDL_GetTicks();
	if(last + interval > now) {
		left = last + interval - now;
		SDL_Delay(left);
		last = now + left;
		return FALSE;
	} else {
		last = now;
		return TRUE;
	}
}

/*
	VA|[gXV(eXg)
*/
void updateSerial(void)
{
	char buf[256];

	if(!serialTest)
		return;

	switch(pin11If) {
	case PIN11IF_3IO:
		sprintf(
		buf,
		"3in 3out [%s %s %s %s %s %s]",
		(io3Out  & 0x01 ? "BUSY": "----"),
		(io3Out  & 0x02 ? "Dout": "----"),
		(pin11In & 0x04 ? "Xin":  "---"),
		(io3Out  & 0x80 ? "Xout": "----"),
		(pin11In & 0x10 ? "Din":  "---"),
		(pin11In & 0x20 ? "ACK":  "---")
		);
		break;
	case PIN11IF_8PIO:
		sprintf(
		buf,
		"PIO [%s %s %s %s %s %s %s %s]",
		(pio8Io & 0x01 ? (pin11In & 0x01 ? "I": "v"): (pio8Out & 0x01 ? "O": "^")),
		(pio8Io & 0x02 ? (pin11In & 0x02 ? "I": "v"): (pio8Out & 0x02 ? "O": "^")),
		(pio8Io & 0x04 ? (pin11In & 0x04 ? "I": "v"): (pio8Out & 0x04 ? "O": "^")),
		(pio8Io & 0x08 ? (pin11In & 0x08 ? "I": "v"): (pio8Out & 0x08 ? "O": "^")),
		(pio8Io & 0x10 ? (pin11In & 0x10 ? "I": "v"): (pio8Out & 0x10 ? "O": "^")),
		(pio8Io & 0x20 ? (pin11In & 0x20 ? "I": "v"): (pio8Out & 0x20 ? "O": "^")),
		(pio8Io & 0x40 ? (pin11In & 0x40 ? "I": "v"): (pio8Out & 0x40 ? "O": "^")),
		(pio8Io & 0x80 ? (pin11In & 0x80 ? "I": "v"): (pio8Out & 0x80 ? "O": "^"))
		);
		break;
	case PIN11IF_UART:
		strcpy(buf, "UART []");
		break;
	default:
		strcpy(buf, "Unknown");
		break;
	}
	SDL_WM_SetCaption(buf, NULL);
}

/*
	o͂
*/
static void SDLCALL playSound(void *unused, Uint8 *stream, int len)
{
	SDL_MixAudio(stream, soundReadBuffer, len, SDL_MIX_MAXVOLUME);
	soundPlayed = TRUE;
}

/*
	ˑI
*/
static SDLCALL void quitDepend(void)
{
	if(buzzer != BUZZER_NONE)
		SDL_CloseAudio();
	SDL_Quit();
}

/*
	ˑ
*/
int initDepend(void)
{
	SDL_Surface *icon;
	Uint8 mask[32 * 32 / 8];

	/* SDL */
	if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | (useJoy ? SDL_INIT_JOYSTICK: 0) | (buzzer != BUZZER_NONE ? SDL_INIT_AUDIO: 0))) {
		fprintf(stderr, "SDL_Init fail. %s\n", SDL_GetError());
		exit(1);
	}
	atexit(quitDepend);
	if((videoInfo = SDL_GetVideoInfo()) == NULL) {
		fprintf(stderr, "SDL_GetVideoInfo fail. %s\n", SDL_GetError());
		exit(1);
	}
	if((icon = SDL_CreateRGBSurfaceFromXpm(g800icon_xpm, mask)) != NULL)
		SDL_WM_SetIcon(icon, mask);
	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
	if(useJoy)
		if((joy = SDL_JoystickOpen(0)) == NULL)
			;
	if(buzzer != BUZZER_NONE) {
		audio.freq     = FREQ_SOUND;
		audio.format   = AUDIO_U8;
		audio.channels = 1;
		audio.samples  = soundBufferSize;
		audio.callback = playSound;
		if(SDL_OpenAudio(&audio, NULL) < 0) {
			fprintf(stderr, "SDL_OpenAudio fail. %s\n", SDL_GetError());
			exit(1);
		}
		SDL_PauseAudio(0);
	}

	/* CAEg */
	updateLayout();

	return 0;
}

/*
	Copyright 2005 ~ 2008 maruhiro
	All rights reserved. 

	Redistribution and use in source and binary forms, 
	with or without modification, are permitted provided that 
	the following conditions are met: 

	 1. Redistributions of source code must retain the above copyright notice, 
	    this list of conditions and the following disclaimer. 

	 2. Redistributions in binary form must reproduce the above copyright notice, 
	    this list of conditions and the following disclaimer in the documentation 
	    and/or other materials provided with the distribution. 

	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
	FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
	THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* eof */
