/*
	SHARP PC-G800 series Emulator
*/

#if !defined(G800_H)
#define G800_H

#include <unistd.h>
#include <stdio.h>
#include "z80.h"
#include "hex.h"
#include "conf.h"

/*
	}N
*/
/* ^U */
#define FALSE	0
#define TRUE	1

/* G~[g}V */
#define MACHINE_FIRST	0
#define MACHINE_E200	0	/* PC-G801/PC-G802/PC-G803/PC-G805/PC-G811/PC-G813/PC-G820/PC-G830/PC-E200/PC-E220 */
#define MACHINE_G815	1	/* PC-G815 */
#define MACHINE_G850	2	/* PC-G850/PC-G850S/PC-G850V */
#define MACHINE_LAST	2

/* L[R[h */
#define GKEY_NONE	0x00
#define GKEY_OFF	0x01
#define GKEY_Q	0x02
#define GKEY_W	0x03
#define GKEY_E	0x04
#define GKEY_R	0x05
#define GKEY_T	0x06
#define GKEY_Y	0x07
#define GKEY_U	0x08
#define GKEY_A	0x09
#define GKEY_S	0x0a
#define GKEY_D	0x0b
#define GKEY_F	0x0c
#define GKEY_G	0x0d
#define GKEY_H	0x0e
#define GKEY_J	0x0f
#define GKEY_K	0x10
#define GKEY_Z	0x11
#define GKEY_X	0x12
#define GKEY_C	0x13
#define GKEY_V	0x14
#define GKEY_B	0x15
#define GKEY_N	0x16
#define GKEY_M	0x17
#define GKEY_COMMA	0x18
#define GKEY_BASIC	0x19
#define GKEY_TEXT	0x1a
#define GKEY_CAPS	0x1b
#define GKEY_KANA	0x1c
#define GKEY_TAB	0x1d
#define GKEY_SPACE	0x1e
#define GKEY_DOWN	0x1f
#define GKEY_UP	0x20
#define GKEY_LEFT	0x21
#define GKEY_RIGHT	0x22
#define GKEY_ANS	0x23
#define GKEY_0	0x24
#define GKEY_PERIOD	0x25
#define GKEY_EQUAL	0x26
#define GKEY_PLUS	0x27
#define GKEY_RETURN	0x28
#define GKEY_L	0x29
#define GKEY_SEMICOLON	0x2a
#define GKEY_CONST	0x2b
#define GKEY_1	0x2c
#define GKEY_2	0x2d
#define GKEY_3	0x2e
#define GKEY_MINUS	0x2f
#define GKEY_MPLUS	0x30
#define GKEY_I	0x31
#define GKEY_O	0x32
#define GKEY_INSERT	0x33
#define GKEY_4	0x34
#define GKEY_5	0x35
#define GKEY_6	0x36
#define GKEY_ASTER	0x37
#define GKEY_RCM	0x38
#define GKEY_P	0x39
#define GKEY_BACKSPACE	0x3a
#define GKEY_PI	0x3b
#define GKEY_7	0x3c
#define GKEY_8	0x3d
#define GKEY_9	0x3e
#define GKEY_SLASH	0x3f
#define GKEY_RKAKKO	0x40
#define GKEY_NPR	0x41
#define GKEY_DEG	0x42
#define GKEY_SQR	0x43
#define GKEY_SQU	0x44
#define GKEY_HAT	0x45
#define GKEY_LKAKKO	0x46
#define GKEY_RCP	0x47
#define GKEY_MDF	0x48
#define GKEY_2NDF	0x49
#define GKEY_SIN	0x4a
#define GKEY_COS	0x4b
#define GKEY_LN	0x4c
#define GKEY_LOG	0x4d
#define GKEY_TAN	0x4e
#define GKEY_FE	0x4f
#define GKEY_CLS	0x50
#define GKEY_BREAK	0x51
#define GKEY_DOUBLE	0x52

/* zL[R[h */
#define GKEY_SHIFT	0x100
#define GKEY_RESET	0x200
#define GKEY_DEBUG	0x300
#define GKEY_11PIN1	0x400
#define GKEY_11PIN2	0x500
#define GKEY_11PIN3	0x600
#define GKEY_11PIN4	0x700
#define GKEY_11PIN5	0x800
#define GKEY_11PIN6	0x900
#define GKEY_11PIN7	0xa00
#define GKEY_11PIN8	0xb00
#define GKEY_MENU	0xc00

/* 荞ݗv */
#define INTERRUPT_IA	0x01
#define INTERRUPT_KON	0x02
#define INTERRUPT_1S	0x04
#define INTERRUPT_INT1	0x08

/* ^C}荞 */
#define TIMER_1S	0x01

/* VA|[g */
#define PIN11_BUSY	0x01	/* [qԍ4  BUSY */
#define PIN11_DOUT	0x02	/* [qԍ5  Dout */
#define PIN11_XIN	0x04	/* [qԍ6  Xin  */
#define PIN11_XOUT	0x08	/* [qԍ7  Xout */
#define PIN11_DIN	0x10	/* [qԍ8  Din  */
#define PIN11_ACK	0x20	/* [qԍ9  ACK  */
#define PIN11_10	0x40	/* [qԍ10 -    */
#define PIN11_11	0x80	/* [qԍ11 -    */

/* ROM(c000h-ffffh)ւ̃|C^ */
#define ROM(page)	&rom[0x4000 * (page)]

/* WCXeBbÑ{^̐ */
#define JOY_BUTTONS	12

/* TEho͂̎g */
#define FREQ_SOUND	44100

/* PC-E200 LCDEVRAM */
#define E200_CELL_WIDTH	5
#define E200_CELL_HEIGHT	7
#define E200_LCD_COLS	24
#define E200_LCD_ROWS	4
#define E200_VRAM_COLS	24
#define E200_VRAM_ROWS	4
#define E200_VRAM_WIDTH	(E200_VRAM_COLS * E200_CELL_WIDTH + 1)
#define E200_VRAM_HEIGHT	(E200_VRAM_ROWS * 8)
#define E200_BUSY_ROW	0
#define E200_BUSY_MASK	0x01
#define E200_CAPS_ROW	0
#define E200_CAPS_MASK	0x02
#define E200_KANA_ROW	0
#define E200_KANA_MASK	0x04
#define E200_SYO_ROW	0
#define E200_SYO_MASK	0x08
#define E200_2NDF_ROW	0
#define E200_2NDF_MASK	0x10
#define E200_TEXT_ROW	3
#define E200_TEXT_MASK	0x08
#define E200_CASL_ROW	3
#define E200_CASL_MASK	0x10
#define E200_PRO_ROW	3
#define E200_PRO_MASK	0x20
#define E200_RUN_ROW	3
#define E200_RUN_MASK	0x40
#define E200_BATT_ROW	4
#define E200_BATT_MASK	0x01
#define E200_E_ROW	4
#define E200_E_MASK	0x02
#define E200_M_ROW	4
#define E200_M_MASK	0x04
#define E200_CONST_ROW	4
#define E200_CONST_MASK	0x08
#define E200_RAD_ROW	4
#define E200_RAD_MASK	0x10
#define E200_G_ROW	4
#define E200_G_MASK	0x20
#define E200_DE_ROW	4
#define E200_DE_MASK	0x40
#define E200_STAT_ROW	7
#define E200_STAT_MASK	0x20
#define E200_PRINT_ROW	7
#define E200_PRINT_MASK	0x40
#define E200_CPU_CLOCKS	(4000 * 1000)
#define E200_STATES_KEYSTROBE_CLEAR	480	/* s */

/* PC-G815 LCDEVRAM */
#define G815_CELL_WIDTH	6
#define G815_CELL_HEIGHT	8
#define G815_LCD_COLS	24
#define G815_LCD_ROWS	4
#define G815_VRAM_COLS	24
#define G815_VRAM_ROWS	4
#define G815_VRAM_WIDTH	(G815_VRAM_COLS * G815_CELL_WIDTH + 1)
#define G815_VRAM_HEIGHT	(G815_VRAM_ROWS * 8)
/* (Xe[^X̎Ȃ) */
#define G815_BUSY_ROW	0
#define G815_BUSY_MASK	0x01
#define G815_CAPS_ROW	0
#define G815_CAPS_MASK	0x02
#define G815_KANA_ROW	0
#define G815_KANA_MASK	0x04
#define G815_SYO_ROW	0
#define G815_SYO_MASK	0x08
#define G815_2NDF_ROW	0
#define G815_2NDF_MASK	0x10
#define G815_TEXT_ROW	3
#define G815_TEXT_MASK	0x08
#define G815_CASL_ROW	3
#define G815_CASL_MASK	0x10
#define G815_PRO_ROW	3
#define G815_PRO_MASK	0x20
#define G815_RUN_ROW	3
#define G815_RUN_MASK	0x40
#define G815_BATT_ROW	4
#define G815_BATT_MASK	0x01
#define G815_E_ROW	4
#define G815_E_MASK	0x02
#define G815_M_ROW	4
#define G815_M_MASK	0x04
#define G815_CONST_ROW	4
#define G815_CONST_MASK	0x08
#define G815_RAD_ROW	4
#define G815_RAD_MASK	0x10
#define G815_G_ROW	4
#define G815_G_MASK	0x20
#define G815_DE_ROW	4
#define G815_DE_MASK	0x40
#define G815_STAT_ROW	7
#define G815_STAT_MASK	0x20
#define G815_PRINT_ROW	7
#define G815_PRINT_MASK	0x40
#define G815_CPU_CLOCKS	(4000 * 1000)
#define G815_STATES_KEYSTROBE_CLEAR	480	/* s */

/* PC-G850 LCDEVRAM */
#define G850_CELL_WIDTH	6
#define G850_CELL_HEIGHT	8
#define G850_LCD_COLS	24
#define G850_LCD_ROWS	6
#define G850_VRAM_COLS	24
#define G850_VRAM_ROWS	8
#define G850_VRAM_WIDTH	(G850_VRAM_COLS * G850_CELL_WIDTH + 1)
#define G850_VRAM_HEIGHT	(G850_VRAM_ROWS * 8)
#define G850_RUN_ROW	0
#define G850_RUN_MASK	0x02
#define G850_PRO_ROW	0
#define G850_PRO_MASK	0x08
#define G850_TEXT_ROW	0
#define G850_TEXT_MASK	0x40
#define G850_CASL_ROW	1
#define G850_CASL_MASK	0x08
#define G850_STAT_ROW	2
#define G850_STAT_MASK	0x01
#define G850_2NDF_ROW	2
#define G850_2NDF_MASK	0x20
#define G850_M_ROW	2
#define G850_M_MASK	0x80
#define G850_CAPS_ROW	3
#define G850_CAPS_MASK	0x04
#define G850_KANA_ROW	3
#define G850_KANA_MASK	0x80
#define G850_SYO_ROW	4
#define G850_SYO_MASK	0x02
#define G850_DE_ROW	4
#define G850_DE_MASK	0x10
#define G850_G_ROW	4
#define G850_G_MASK	0x40
#define G850_RAD_ROW	5
#define G850_RAD_MASK	0x01
#define G850_CONST_ROW	5
#define G850_CONST_MASK	0x04
#define G850_PRINT_ROW	5
#define G850_PRINT_MASK	0x10
#define G850_BUSY_ROW	5
#define G850_BUSY_MASK	0x40
#define G850_BATT_ROW	5
#define G850_BATT_MASK	0x80
#define G850_CPU_CLOCKS	(9000 * 1000)
#define G850_STATES_KEYSTROBE_CLEAR	130

/* CAEg */
#define PIXEL_WIDTH	10
#define PIXEL_HEIGHT	10
#define STATUS_CHAR_WIDTH	30
#define STATUS_HEIGHT	48
#define STATUS_RUN_WIDTH	(3 * STATUS_CHAR_WIDTH)
#define STATUS_PRO_WIDTH	(3 * STATUS_CHAR_WIDTH)
#define STATUS_TEXT_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_CASL_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_STAT_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_2NDF_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_M_WIDTH	(2 * STATUS_CHAR_WIDTH)
#define STATUS_CAPS_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_KANA_WIDTH	(3 * STATUS_CHAR_WIDTH)
#define STATUS_SYO_WIDTH	(2 * STATUS_CHAR_WIDTH)
#define STATUS_DE_WIDTH	(2 * STATUS_CHAR_WIDTH)
#define STATUS_G_WIDTH	(1 * STATUS_CHAR_WIDTH)
#define STATUS_RAD_WIDTH	(3 * STATUS_CHAR_WIDTH)
#define STATUS_CONST_WIDTH	(5 * STATUS_CHAR_WIDTH)
#define STATUS_PRINT_WIDTH	(5 * STATUS_CHAR_WIDTH)
#define STATUS_BUSY_WIDTH	(4 * STATUS_CHAR_WIDTH)
#define STATUS_BATT_WIDTH	(5 * STATUS_CHAR_WIDTH)
#define STATUS_E_WIDTH	(1 * STATUS_CHAR_WIDTH)
#define STATUS_FIRST	0
#define STATUS_RUN	0
#define STATUS_PRO	1
#define STATUS_TEXT	2
#define STATUS_CASL	3
#define STATUS_STAT	4
#define STATUS_2NDF	5
#define STATUS_M	6
#define STATUS_CAPS	7
#define STATUS_KANA	8
#define STATUS_SYO	9
#define STATUS_DE	10
#define STATUS_G	11
#define STATUS_RAD	12
#define STATUS_CONST	13
#define STATUS_PRINT	14
#define STATUS_BUSY	15
#define STATUS_BATT	16
#define STATUS_E	17
#define STATUS_LAST	17
#define KEY_WIDTH	80
#define KEY_HEIGHT	60

/* CAEg PC-E200 */
#define E200_MARGIN_UP	60
#define E200_MARGIN_DOWN	(60 - PIXEL_HEIGHT)
#define E200_MARGIN_LEFT	PIXEL_WIDTH
#define E200_MARGIN_RIGHT	0
#define E200_LCD_MAIN_WIDTH	((E200_CELL_WIDTH + 1) * E200_LCD_COLS * PIXEL_WIDTH)
#define E200_LCD_MAIN_HEIGHT	((E200_CELL_HEIGHT + 1) * E200_LCD_ROWS * PIXEL_HEIGHT)
#define E200_LCD_X	0
#define E200_LCD_Y	0
#define E200_LCD_WIDTH	(E200_MARGIN_LEFT + E200_LCD_MAIN_WIDTH + E200_MARGIN_RIGHT)
#define E200_LCD_HEIGHT	(E200_MARGIN_UP + E200_LCD_MAIN_HEIGHT + E200_MARGIN_DOWN + PIXEL_HEIGHT)
#define _E200_STATUS_X	E200_MARGIN_LEFT
#define _E200_USTATUS_Y	(E200_MARGIN_UP - STATUS_HEIGHT - PIXEL_HEIGHT)
#define _E200_DSTATUS_Y	(E200_MARGIN_UP + E200_LCD_MAIN_HEIGHT + PIXEL_HEIGHT)
#define E200_STATUS_RUN_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 2)
#define E200_STATUS_RUN_Y	_E200_DSTATUS_Y
#define E200_STATUS_PRO_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 4)
#define E200_STATUS_PRO_Y	_E200_DSTATUS_Y
#define E200_STATUS_TEXT_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 9)
#define E200_STATUS_TEXT_Y	_E200_DSTATUS_Y
#define E200_STATUS_CASL_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 6)
#define E200_STATUS_CASL_Y	_E200_DSTATUS_Y
#define E200_STATUS_STAT_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 18)
#define E200_STATUS_STAT_Y	_E200_DSTATUS_Y
#define E200_STATUS_2NDF_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 11)
#define E200_STATUS_2NDF_Y	_E200_USTATUS_Y
#define E200_STATUS_M_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 19)
#define E200_STATUS_M_Y	_E200_USTATUS_Y
#define E200_STATUS_CAPS_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 3)
#define E200_STATUS_CAPS_Y	_E200_USTATUS_Y
#define E200_STATUS_KANA_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 5)
#define E200_STATUS_KANA_Y	_E200_USTATUS_Y
#define E200_STATUS_SYO_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 7)
#define E200_STATUS_SYO_Y	_E200_USTATUS_Y
#define E200_STATUS_DE_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 13)
#define E200_STATUS_DE_Y	_E200_USTATUS_Y
#define E200_STATUS_G_X	(E200_STATUS_DE_X + STATUS_DE_WIDTH)
#define E200_STATUS_G_Y	_E200_USTATUS_Y
#define E200_STATUS_RAD_X	(E200_STATUS_G_X + STATUS_G_WIDTH)
#define E200_STATUS_RAD_Y	_E200_USTATUS_Y
#define E200_STATUS_CONST_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 16)
#define E200_STATUS_CONST_Y	_E200_USTATUS_Y
#define E200_STATUS_PRINT_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 20)
#define E200_STATUS_PRINT_Y	_E200_DSTATUS_Y
#define E200_STATUS_BUSY_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 0)
#define E200_STATUS_BUSY_Y	_E200_USTATUS_Y
#define E200_STATUS_BATT_X	(E200_MARGIN_LEFT + E200_LCD_MAIN_WIDTH - STATUS_BATT_WIDTH)
#define E200_STATUS_BATT_Y	_E200_USTATUS_Y
#define E200_STATUS_E_X	(_E200_STATUS_X + PIXEL_WIDTH * 6 * 21)
#define E200_STATUS_E_Y	_E200_USTATUS_Y
#define E200_FKEY_X	(E200_LCD_X + E200_LCD_WIDTH + 20)
#define E200_FKEY_Y	E200_MARGIN_UP
#define E200_FKEY_WIDTH	100
#define E200_FKEY_HEIGHT	120
#define E200_FUNCTION_KEY_X	E200_FKEY_X
#define E200_FUNCTION_KEY_Y	E200_FKEY_Y
#define E200_FUNCTION_KEY_WIDTH	(E200_FKEY_WIDTH * 9 + 20)
#define E200_FUNCTION_KEY_HEIGHT	(E200_FKEY_HEIGHT * 3)
#define E200_BASIC_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 0)
#define E200_BASIC_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_TEXT_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 1)
#define E200_TEXT_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_CONST_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 2)
#define E200_CONST_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_ANS_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 2)
#define E200_ANS_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_OFF_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 0)
#define E200_OFF_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_BREAK_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 0)
#define E200_BREAK_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_2NDF_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 3)
#define E200_2NDF_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_SIN_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 4)
#define E200_SIN_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_COS_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 5)
#define E200_COS_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_TAN_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 6)
#define E200_TAN_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_FE_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 7)
#define E200_FE_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_CLS_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 8)
#define E200_CLS_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 0)
#define E200_NPR_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 3)
#define E200_NPR_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_DEG_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 4)
#define E200_DEG_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_LN_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 5)
#define E200_LN_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_LOG_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 6)
#define E200_LOG_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_RCP_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 7)
#define E200_RCP_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_MDF_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 8)
#define E200_MDF_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 1)
#define E200_PI_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 3)
#define E200_PI_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_SQR_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 4)
#define E200_SQR_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_SQU_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 5)
#define E200_SQU_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_HAT_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 6)
#define E200_HAT_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_LKAKKO_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 7)
#define E200_LKAKKO_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)
#define E200_RKAKKO_X	(E200_FKEY_X + E200_FKEY_WIDTH  * 8)
#define E200_RKAKKO_Y	(E200_FKEY_Y + E200_FKEY_HEIGHT * 2)

/* CAEg PC-G815 */
#define G815_MARGIN_UP	E200_MARGIN_UP
#define G815_MARGIN_DOWN	E200_MARGIN_DOWN
#define G815_MARGIN_LEFT	E200_MARGIN_LEFT
#define G815_MARGIN_RIGHT	(E200_MARGIN_RIGHT + PIXEL_WIDTH)
#define G815_LCD_MAIN_WIDTH	E200_LCD_MAIN_WIDTH
#define G815_LCD_MAIN_HEIGHT	E200_LCD_MAIN_HEIGHT
#define G815_LCD_X	0
#define G815_LCD_Y	0
#define G815_LCD_WIDTH	(G815_MARGIN_LEFT + G815_LCD_MAIN_WIDTH + G815_MARGIN_RIGHT)
#define G815_LCD_HEIGHT	(G815_MARGIN_UP + G815_LCD_MAIN_HEIGHT + G815_MARGIN_DOWN + PIXEL_HEIGHT)
#define _G815_STATUS_X	G815_MARGIN_LEFT
#define _G815_USTATUS_Y	(G815_MARGIN_UP - STATUS_HEIGHT - PIXEL_HEIGHT)
#define _G815_DSTATUS_Y	(G815_MARGIN_UP + G815_LCD_MAIN_HEIGHT + PIXEL_HEIGHT)
#define G815_STATUS_RUN_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 2)
#define G815_STATUS_RUN_Y	_G815_DSTATUS_Y
#define G815_STATUS_PRO_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 4)
#define G815_STATUS_PRO_Y	_G815_DSTATUS_Y
#define G815_STATUS_TEXT_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 9)
#define G815_STATUS_TEXT_Y	_G815_DSTATUS_Y
#define G815_STATUS_CASL_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 6)
#define G815_STATUS_CASL_Y	_G815_DSTATUS_Y
#define G815_STATUS_STAT_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 18)
#define G815_STATUS_STAT_Y	_G815_DSTATUS_Y
#define G815_STATUS_2NDF_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 11)
#define G815_STATUS_2NDF_Y	_G815_USTATUS_Y
#define G815_STATUS_M_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 19)
#define G815_STATUS_M_Y	_G815_USTATUS_Y
#define G815_STATUS_CAPS_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 3)
#define G815_STATUS_CAPS_Y	_G815_USTATUS_Y
#define G815_STATUS_KANA_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 5)
#define G815_STATUS_KANA_Y	_G815_USTATUS_Y
#define G815_STATUS_SYO_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 7)
#define G815_STATUS_SYO_Y	_G815_USTATUS_Y
#define G815_STATUS_DE_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 13)
#define G815_STATUS_DE_Y	_G815_USTATUS_Y
#define G815_STATUS_G_X	(G815_STATUS_DE_X + STATUS_DE_WIDTH)
#define G815_STATUS_G_Y	_G815_USTATUS_Y
#define G815_STATUS_RAD_X	(G815_STATUS_G_X + STATUS_G_WIDTH)
#define G815_STATUS_RAD_Y	_G815_USTATUS_Y
#define G815_STATUS_CONST_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 16)
#define G815_STATUS_CONST_Y	_G815_USTATUS_Y
#define G815_STATUS_PRINT_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 20)
#define G815_STATUS_PRINT_Y	_G815_DSTATUS_Y
#define G815_STATUS_BUSY_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 0)
#define G815_STATUS_BUSY_Y	_G815_USTATUS_Y
#define G815_STATUS_BATT_X	(G815_MARGIN_LEFT + G815_LCD_MAIN_WIDTH - STATUS_BATT_WIDTH)
#define G815_STATUS_BATT_Y	_G815_USTATUS_Y
#define G815_STATUS_E_X	(_G815_STATUS_X + PIXEL_WIDTH * 6 * 21)
#define G815_STATUS_E_Y	_G815_USTATUS_Y
#define G815_FKEY_X	(G815_LCD_X + G815_LCD_WIDTH + 20)
#define G815_FKEY_Y	G815_MARGIN_UP
#define G815_FKEY_WIDTH	100
#define G815_FKEY_HEIGHT	120
#define G815_FUNCTION_KEY_X	G815_FKEY_X
#define G815_FUNCTION_KEY_Y	G815_FKEY_Y
#define G815_FUNCTION_KEY_WIDTH	(G815_FKEY_WIDTH * 9 + 20)
#define G815_FUNCTION_KEY_HEIGHT	(G815_FKEY_HEIGHT * 3)
#define G815_BASIC_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 0)
#define G815_BASIC_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_TEXT_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 1)
#define G815_TEXT_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_CONST_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 2)
#define G815_CONST_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_ANS_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 2)
#define G815_ANS_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_OFF_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 0)
#define G815_OFF_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_BREAK_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 0)
#define G815_BREAK_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_2NDF_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 3)
#define G815_2NDF_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_SIN_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 4)
#define G815_SIN_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_COS_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 5)
#define G815_COS_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_TAN_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 6)
#define G815_TAN_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_FE_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 7)
#define G815_FE_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_CLS_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 8)
#define G815_CLS_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 0)
#define G815_NPR_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 3)
#define G815_NPR_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_DEG_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 4)
#define G815_DEG_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_LN_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 5)
#define G815_LN_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_LOG_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 6)
#define G815_LOG_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_RCP_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 7)
#define G815_RCP_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_MDF_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 8)
#define G815_MDF_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 1)
#define G815_PI_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 3)
#define G815_PI_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_SQR_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 4)
#define G815_SQR_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_SQU_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 5)
#define G815_SQU_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_HAT_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 6)
#define G815_HAT_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_LKAKKO_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 7)
#define G815_LKAKKO_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)
#define G815_RKAKKO_X	(G815_FKEY_X + G815_FKEY_WIDTH  * 8)
#define G815_RKAKKO_Y	(G815_FKEY_Y + G815_FKEY_HEIGHT * 2)

/* CAEg PC-G850 */
#define G850_MARGIN_UP	20
#define G850_MARGIN_DOWN	20
#define G850_MARGIN_LEFT	180
#define G850_MARGIN_RIGHT	200
#define G850_LCD_MAIN_WIDTH	(G850_CELL_WIDTH * G850_LCD_COLS * PIXEL_WIDTH)
#define G850_LCD_MAIN_HEIGHT	(G850_CELL_HEIGHT * G850_LCD_ROWS * PIXEL_HEIGHT)
#define G850_LCD_X	0
#define G850_LCD_Y	0
#define G850_LCD_WIDTH	(G850_MARGIN_LEFT + G850_LCD_MAIN_WIDTH + G850_MARGIN_RIGHT)
#define G850_LCD_HEIGHT	(G850_MARGIN_UP + G850_LCD_MAIN_HEIGHT + G850_MARGIN_DOWN)
#define _G850_RSTATUS_X	(G850_MARGIN_LEFT + G850_LCD_MAIN_WIDTH + PIXEL_WIDTH)
#define _G850_LSTATUS_X	PIXEL_WIDTH
#define _G850_STATUS_Y	G850_MARGIN_UP
#define G850_STATUS_RUN_X	_G850_RSTATUS_X
#define G850_STATUS_RUN_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 0)
#define G850_STATUS_PRO_X	(G850_STATUS_RUN_X + STATUS_RUN_WIDTH)
#define G850_STATUS_PRO_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 0)
#define G850_STATUS_TEXT_X	_G850_RSTATUS_X
#define G850_STATUS_TEXT_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 1)
#define G850_STATUS_CASL_X	_G850_RSTATUS_X
#define G850_STATUS_CASL_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 2)
#define G850_STATUS_STAT_X	_G850_RSTATUS_X
#define G850_STATUS_STAT_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 3)
#define G850_STATUS_2NDF_X	_G850_RSTATUS_X
#define G850_STATUS_2NDF_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 4)
#define G850_STATUS_M_X	(G850_STATUS_2NDF_X + STATUS_2NDF_WIDTH)
#define G850_STATUS_M_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 4)
#define G850_STATUS_CAPS_X	_G850_RSTATUS_X
#define G850_STATUS_CAPS_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 5)
#define G850_STATUS_KANA_X	_G850_RSTATUS_X
#define G850_STATUS_KANA_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 6)
#define G850_STATUS_SYO_X	(G850_STATUS_KANA_X + STATUS_KANA_WIDTH)
#define G850_STATUS_SYO_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 6)
#define G850_STATUS_DE_X	_G850_RSTATUS_X
#define G850_STATUS_DE_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 7)
#define G850_STATUS_G_X	(G850_STATUS_DE_X + STATUS_DE_WIDTH)
#define G850_STATUS_G_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 7)
#define G850_STATUS_RAD_X	(G850_STATUS_G_X + STATUS_G_WIDTH)
#define G850_STATUS_RAD_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 7)
#define G850_STATUS_CONST_X	_G850_RSTATUS_X
#define G850_STATUS_CONST_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 8)
#define G850_STATUS_PRINT_X	_G850_RSTATUS_X
#define G850_STATUS_PRINT_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 9)
#define G850_STATUS_BUSY_X	_G850_LSTATUS_X
#define G850_STATUS_BUSY_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 8)
#define G850_STATUS_BATT_X	_G850_LSTATUS_X
#define G850_STATUS_BATT_Y	(_G850_STATUS_Y + STATUS_HEIGHT * 9)
#define G850_FKEY_X	(G850_LCD_X + G850_LCD_WIDTH + 20)
#define G850_FKEY_Y	(G850_LCD_Y + 40)
#define G850_FKEY_WIDTH	100
#define G850_FKEY_HEIGHT	120
#define G850_FUNCTION_KEY_X	G850_FKEY_X
#define G850_FUNCTION_KEY_Y	G850_FKEY_Y
#define G850_FUNCTION_KEY_WIDTH	(G850_FKEY_WIDTH * 6 + 20)
#define G850_FUNCTION_KEY_HEIGHT	(G850_FKEY_HEIGHT * 4)
#define G850_BASIC_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 0)
#define G850_BASIC_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_TEXT_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 1)
#define G850_TEXT_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_CONST_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 2)
#define G850_CONST_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_ANS_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 3)
#define G850_ANS_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_OFF_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 4)
#define G850_OFF_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_BREAK_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 5)
#define G850_BREAK_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 0)
#define G850_2NDF_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 0)
#define G850_2NDF_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_SIN_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 1)
#define G850_SIN_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_COS_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 2)
#define G850_COS_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_TAN_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 3)
#define G850_TAN_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_FE_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 4)
#define G850_FE_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_CLS_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 5)
#define G850_CLS_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 1)
#define G850_NPR_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 0)
#define G850_NPR_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_DEG_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 1)
#define G850_DEG_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_LN_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 2)
#define G850_LN_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_LOG_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 3)
#define G850_LOG_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_RCP_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 4)
#define G850_RCP_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_MDF_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 5)
#define G850_MDF_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 2)
#define G850_PI_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 0)
#define G850_PI_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)
#define G850_SQR_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 1)
#define G850_SQR_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)
#define G850_SQU_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 2)
#define G850_SQU_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)
#define G850_HAT_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 3)
#define G850_HAT_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)
#define G850_LKAKKO_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 4)
#define G850_LKAKKO_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)
#define G850_RKAKKO_X	(G850_FKEY_X + G850_FKEY_WIDTH  * 5)
#define G850_RKAKKO_Y	(G850_FKEY_Y + G850_FKEY_HEIGHT * 3)

/*
	\
*/
/* ` */
typedef struct {
	int x;
	int y;
	int w;
	int h;
} Rect;

/* rbg}bv摜 */
typedef struct {
	int width;
	int height;
	uint8 *image;
} Bitmap;

/* }V */
typedef struct {
	int machine;
	char *machine_name;
	int cpu_clocks;
	int states_keystrobe_clear;
	int cell_width;
	int cell_height;
	int lcd_cols;
	int lcd_rows;
	int vram_width;
	int vram_height;
	int vram_cols;
	int vram_rows;
	int status_row[STATUS_LAST + 1];
	uint8 status_mask[STATUS_LAST + 1];
	Rect pos_lcd;
	Rect pos_lcd_main;
	Rect pos_lcd_status[STATUS_LAST + 1];
	Rect pos_function_key;
	Rect pos_key[0x52];
} Machineinfo;

/*
	O[oϐ
*/
#if defined(DEF_GLOBAL)
#	define EXTERN
#else
#	define EXTERN extern
#endif

/* PC-E200 */
EXTERN const Machineinfo e200info
#if defined(DEF_GLOBAL)
= {
	MACHINE_E200,
	"PC-E200",
	E200_CPU_CLOCKS,
	E200_STATES_KEYSTROBE_CLEAR,
	E200_CELL_WIDTH, E200_CELL_HEIGHT,
	E200_LCD_COLS,   E200_LCD_ROWS,
	E200_VRAM_WIDTH, E200_VRAM_HEIGHT,
	E200_VRAM_COLS,  E200_VRAM_ROWS,
	{
		E200_RUN_ROW,  E200_PRO_ROW,   E200_TEXT_ROW,  E200_CASL_ROW,
		E200_STAT_ROW, E200_2NDF_ROW,  E200_M_ROW,     E200_CAPS_ROW,
		E200_KANA_ROW, E200_SYO_ROW,   E200_DE_ROW,    E200_G_ROW,
		E200_RAD_ROW,  E200_CONST_ROW, E200_PRINT_ROW, E200_BUSY_ROW,
		E200_BATT_ROW, E200_E_ROW
	}, {
		E200_RUN_MASK,  E200_PRO_MASK,   E200_TEXT_MASK,  E200_CASL_MASK,
		E200_STAT_MASK, E200_2NDF_MASK,  E200_M_MASK,     E200_CAPS_MASK,
		E200_KANA_MASK, E200_SYO_MASK,   E200_DE_MASK,    E200_G_MASK,
		E200_RAD_MASK,  E200_CONST_MASK, E200_PRINT_MASK, E200_BUSY_MASK,
		E200_BATT_MASK, E200_E_MASK
	}, {
		E200_LCD_X, E200_LCD_Y, E200_LCD_WIDTH, E200_LCD_HEIGHT
	}, {
		E200_MARGIN_LEFT, E200_MARGIN_UP, E200_LCD_MAIN_WIDTH, E200_LCD_MAIN_HEIGHT
	}, {
		{ E200_STATUS_RUN_X,   E200_STATUS_RUN_Y,   STATUS_RUN_WIDTH,   STATUS_HEIGHT },
		{ E200_STATUS_PRO_X,   E200_STATUS_PRO_Y,   STATUS_PRO_WIDTH,   STATUS_HEIGHT },
		{ E200_STATUS_TEXT_X,  E200_STATUS_TEXT_Y,  STATUS_TEXT_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_CASL_X,  E200_STATUS_CASL_Y,  STATUS_CASL_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_STAT_X,  E200_STATUS_STAT_Y,  STATUS_STAT_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_2NDF_X,  E200_STATUS_2NDF_Y,  STATUS_2NDF_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_M_X,     E200_STATUS_M_Y,     STATUS_M_WIDTH,     STATUS_HEIGHT },
		{ E200_STATUS_CAPS_X,  E200_STATUS_CAPS_Y,  STATUS_CAPS_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_KANA_X,  E200_STATUS_KANA_Y,  STATUS_KANA_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_SYO_X,   E200_STATUS_SYO_Y,   STATUS_SYO_WIDTH,   STATUS_HEIGHT },
		{ E200_STATUS_DE_X,    E200_STATUS_DE_Y,    STATUS_DE_WIDTH,    STATUS_HEIGHT },
		{ E200_STATUS_G_X,     E200_STATUS_G_Y,     STATUS_G_WIDTH,     STATUS_HEIGHT },
		{ E200_STATUS_RAD_X,   E200_STATUS_RAD_Y,   STATUS_RAD_WIDTH,   STATUS_HEIGHT },
		{ E200_STATUS_CONST_X, E200_STATUS_CONST_Y, STATUS_CONST_WIDTH, STATUS_HEIGHT },
		{ E200_STATUS_PRINT_X, E200_STATUS_PRINT_Y, STATUS_PRINT_WIDTH, STATUS_HEIGHT },
		{ E200_STATUS_BUSY_X,  E200_STATUS_BUSY_Y,  STATUS_BUSY_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_BATT_X,  E200_STATUS_BATT_Y,  STATUS_BATT_WIDTH,  STATUS_HEIGHT },
		{ E200_STATUS_E_X,     E200_STATUS_E_Y,     STATUS_E_WIDTH,     STATUS_HEIGHT }
	}, {
		E200_FUNCTION_KEY_X, E200_FUNCTION_KEY_Y, E200_FUNCTION_KEY_WIDTH, E200_FUNCTION_KEY_HEIGHT
	}, {
		{ 0, 0, 0, 0 },
		{ E200_OFF_X, E200_OFF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ E200_BASIC_X, E200_BASIC_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_TEXT_X,  E200_TEXT_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ E200_ANS_X, E200_ANS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ E200_CONST_X, E200_CONST_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ E200_PI_X, E200_PI_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ E200_RKAKKO_X, E200_RKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_NPR_X, E200_NPR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_DEG_X, E200_DEG_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_SQR_X, E200_SQR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_SQU_X, E200_SQU_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_HAT_X, E200_HAT_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_LKAKKO_X, E200_LKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_RCP_X, E200_RCP_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_MDF_X, E200_MDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_2NDF_X, E200_2NDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_SIN_X,  E200_SIN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ E200_COS_X,  E200_COS_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ E200_LN_X,   E200_LN_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ E200_LOG_X,  E200_LOG_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ E200_TAN_X,  E200_TAN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ E200_FE_X,   E200_FE_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ E200_CLS_X, E200_CLS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ E200_BREAK_X, E200_BREAK_Y, KEY_WIDTH, KEY_HEIGHT }
	}
}
#endif
;

/* PC-G815 */
EXTERN const Machineinfo g815info
#if defined(DEF_GLOBAL)
= {
	MACHINE_G815,
	"PC-G815",
	G815_CPU_CLOCKS,
	G815_STATES_KEYSTROBE_CLEAR,
	G815_CELL_WIDTH, G815_CELL_HEIGHT,
	G815_LCD_COLS,   G815_LCD_ROWS,
	G815_VRAM_WIDTH, G815_VRAM_HEIGHT,
	G815_VRAM_COLS,  G815_VRAM_ROWS,
	{
		G815_RUN_ROW,  G815_PRO_ROW,   G815_TEXT_ROW,  G815_CASL_ROW,
		G815_STAT_ROW, G815_2NDF_ROW,  G815_M_ROW,     G815_CAPS_ROW,
		G815_KANA_ROW, G815_SYO_ROW,   G815_DE_ROW,    G815_G_ROW,
		G815_RAD_ROW,  G815_CONST_ROW, G815_PRINT_ROW, G815_BUSY_ROW,
		G815_BATT_ROW, G815_E_ROW
	}, {
		G815_RUN_MASK,  G815_PRO_MASK,   G815_TEXT_MASK,  G815_CASL_MASK,
		G815_STAT_MASK, G815_2NDF_MASK,  G815_M_MASK,     G815_CAPS_MASK,
		G815_KANA_MASK, G815_SYO_MASK,   G815_DE_MASK,    G815_G_MASK,
		G815_RAD_MASK,  G815_CONST_MASK, G815_PRINT_MASK, G815_BUSY_MASK,
		G815_BATT_MASK, G815_E_MASK
	}, {
		G815_LCD_X, G815_LCD_Y, G815_LCD_WIDTH, G815_LCD_HEIGHT
	}, {
		G815_MARGIN_LEFT, G815_MARGIN_UP, G815_LCD_MAIN_WIDTH, G815_LCD_MAIN_HEIGHT
	}, {
		{ G815_STATUS_RUN_X,   G815_STATUS_RUN_Y,   STATUS_RUN_WIDTH,   STATUS_HEIGHT },
		{ G815_STATUS_PRO_X,   G815_STATUS_PRO_Y,   STATUS_PRO_WIDTH,   STATUS_HEIGHT },
		{ G815_STATUS_TEXT_X,  G815_STATUS_TEXT_Y,  STATUS_TEXT_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_CASL_X,  G815_STATUS_CASL_Y,  STATUS_CASL_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_STAT_X,  G815_STATUS_STAT_Y,  STATUS_STAT_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_2NDF_X,  G815_STATUS_2NDF_Y,  STATUS_2NDF_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_M_X,     G815_STATUS_M_Y,     STATUS_M_WIDTH,     STATUS_HEIGHT },
		{ G815_STATUS_CAPS_X,  G815_STATUS_CAPS_Y,  STATUS_CAPS_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_KANA_X,  G815_STATUS_KANA_Y,  STATUS_KANA_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_SYO_X,   G815_STATUS_SYO_Y,   STATUS_SYO_WIDTH,   STATUS_HEIGHT },
		{ G815_STATUS_DE_X,    G815_STATUS_DE_Y,    STATUS_DE_WIDTH,    STATUS_HEIGHT },
		{ G815_STATUS_G_X,     G815_STATUS_G_Y,     STATUS_G_WIDTH,     STATUS_HEIGHT },
		{ G815_STATUS_RAD_X,   G815_STATUS_RAD_Y,   STATUS_RAD_WIDTH,   STATUS_HEIGHT },
		{ G815_STATUS_CONST_X, G815_STATUS_CONST_Y, STATUS_CONST_WIDTH, STATUS_HEIGHT },
		{ G815_STATUS_PRINT_X, G815_STATUS_PRINT_Y, STATUS_PRINT_WIDTH, STATUS_HEIGHT },
		{ G815_STATUS_BUSY_X,  G815_STATUS_BUSY_Y,  STATUS_BUSY_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_BATT_X,  G815_STATUS_BATT_Y,  STATUS_BATT_WIDTH,  STATUS_HEIGHT },
		{ G815_STATUS_E_X,     G815_STATUS_E_Y,     STATUS_E_WIDTH,     STATUS_HEIGHT }
	}, {
		G815_FUNCTION_KEY_X, G815_FUNCTION_KEY_Y, G815_FUNCTION_KEY_WIDTH, G815_FUNCTION_KEY_HEIGHT
	}, {
		{ 0, 0, 0, 0 },
		{ G815_OFF_X, G815_OFF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G815_BASIC_X, G815_BASIC_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_TEXT_X,  G815_TEXT_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G815_ANS_X, G815_ANS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G815_CONST_X, G815_CONST_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G815_PI_X, G815_PI_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G815_RKAKKO_X, G815_RKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_NPR_X, G815_NPR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_DEG_X, G815_DEG_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_SQR_X, G815_SQR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_SQU_X, G815_SQU_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_HAT_X, G815_HAT_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_LKAKKO_X, G815_LKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_RCP_X, G815_RCP_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_MDF_X, G815_MDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_2NDF_X, G815_2NDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_SIN_X,  G815_SIN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G815_COS_X,  G815_COS_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G815_LN_X,   G815_LN_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ G815_LOG_X,  G815_LOG_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G815_TAN_X,  G815_TAN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G815_FE_X,   G815_FE_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ G815_CLS_X, G815_CLS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G815_BREAK_X, G815_BREAK_Y, KEY_WIDTH, KEY_HEIGHT }
	}
}
#endif
;

/* PC-G850 */
EXTERN const Machineinfo g850info
#if defined(DEF_GLOBAL)
= {
	MACHINE_G850,
	"PC-G850",
	G850_CPU_CLOCKS,
	G850_STATES_KEYSTROBE_CLEAR,
	G850_CELL_WIDTH, G850_CELL_HEIGHT,
	G850_LCD_COLS,   G850_LCD_ROWS,
	G850_VRAM_WIDTH, G850_VRAM_HEIGHT,
	G850_VRAM_COLS,  G850_VRAM_ROWS,
	{
		G850_RUN_ROW,  G850_PRO_ROW,   G850_TEXT_ROW,  G850_CASL_ROW,
		G850_STAT_ROW, G850_2NDF_ROW,  G850_M_ROW,     G850_CAPS_ROW,
		G850_KANA_ROW, G850_SYO_ROW,   G850_DE_ROW,    G850_G_ROW,
		G850_RAD_ROW,  G850_CONST_ROW, G850_PRINT_ROW, G850_BUSY_ROW,
		G850_BATT_ROW, 0
	}, {
		G850_RUN_MASK,  G850_PRO_MASK,   G850_TEXT_MASK,  G850_CASL_MASK,
		G850_STAT_MASK, G850_2NDF_MASK,  G850_M_MASK,     G850_CAPS_MASK,
		G850_KANA_MASK, G850_SYO_MASK,   G850_DE_MASK,    G850_G_MASK,
		G850_RAD_MASK,  G850_CONST_MASK, G850_PRINT_MASK, G850_BUSY_MASK,
		G850_BATT_MASK, 0
	}, {
		G850_LCD_X, G850_LCD_Y, G850_LCD_WIDTH, G850_LCD_HEIGHT
	}, {
		G850_MARGIN_LEFT, G850_MARGIN_UP, G850_LCD_MAIN_WIDTH, G850_LCD_MAIN_HEIGHT
	}, {
		{ G850_STATUS_RUN_X,   G850_STATUS_RUN_Y,   STATUS_RUN_WIDTH,   STATUS_HEIGHT },
		{ G850_STATUS_PRO_X,   G850_STATUS_PRO_Y,   STATUS_PRO_WIDTH,   STATUS_HEIGHT },
		{ G850_STATUS_TEXT_X,  G850_STATUS_TEXT_Y,  STATUS_TEXT_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_CASL_X,  G850_STATUS_CASL_Y,  STATUS_CASL_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_STAT_X,  G850_STATUS_STAT_Y,  STATUS_STAT_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_2NDF_X,  G850_STATUS_2NDF_Y,  STATUS_2NDF_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_M_X,     G850_STATUS_M_Y,     STATUS_M_WIDTH,     STATUS_HEIGHT },
		{ G850_STATUS_CAPS_X,  G850_STATUS_CAPS_Y,  STATUS_CAPS_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_KANA_X,  G850_STATUS_KANA_Y,  STATUS_KANA_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_SYO_X,   G850_STATUS_SYO_Y,   STATUS_SYO_WIDTH,   STATUS_HEIGHT },
		{ G850_STATUS_DE_X,    G850_STATUS_DE_Y,    STATUS_DE_WIDTH,    STATUS_HEIGHT },
		{ G850_STATUS_G_X,     G850_STATUS_G_Y,     STATUS_G_WIDTH,     STATUS_HEIGHT },
		{ G850_STATUS_RAD_X,   G850_STATUS_RAD_Y,   STATUS_RAD_WIDTH,   STATUS_HEIGHT },
		{ G850_STATUS_CONST_X, G850_STATUS_CONST_Y, STATUS_CONST_WIDTH, STATUS_HEIGHT },
		{ G850_STATUS_PRINT_X, G850_STATUS_PRINT_Y, STATUS_PRINT_WIDTH, STATUS_HEIGHT },
		{ G850_STATUS_BUSY_X,  G850_STATUS_BUSY_Y,  STATUS_BUSY_WIDTH,  STATUS_HEIGHT },
		{ G850_STATUS_BATT_X,  G850_STATUS_BATT_Y,  STATUS_BATT_WIDTH,  STATUS_HEIGHT },
		{ 0, 0, 0, 0 }
	}, {
		G850_FUNCTION_KEY_X, G850_FUNCTION_KEY_Y, G850_FUNCTION_KEY_WIDTH, G850_FUNCTION_KEY_HEIGHT
	}, {
		{ 0, 0, 0, 0 },
		{ G850_OFF_X, G850_OFF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G850_BASIC_X, G850_BASIC_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_TEXT_X,  G850_TEXT_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G850_ANS_X, G850_ANS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G850_CONST_X, G850_CONST_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G850_PI_X, G850_PI_Y, KEY_WIDTH, KEY_HEIGHT },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ 0, 0, 0, 0 },
		{ G850_RKAKKO_X, G850_RKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_NPR_X, G850_NPR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_DEG_X, G850_DEG_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_SQR_X, G850_SQR_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_SQU_X, G850_SQU_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_HAT_X, G850_HAT_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_LKAKKO_X, G850_LKAKKO_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_RCP_X, G850_RCP_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_MDF_X, G850_MDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_2NDF_X, G850_2NDF_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_SIN_X,  G850_SIN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G850_COS_X,  G850_COS_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G850_LN_X,   G850_LN_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ G850_LOG_X,  G850_LOG_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G850_TAN_X,  G850_TAN_Y,  KEY_WIDTH, KEY_HEIGHT },
		{ G850_FE_X,   G850_FE_Y,   KEY_WIDTH, KEY_HEIGHT },
		{ G850_CLS_X, G850_CLS_Y, KEY_WIDTH, KEY_HEIGHT },
		{ G850_BREAK_X, G850_BREAK_Y, KEY_WIDTH, KEY_HEIGHT }
	}
}
#endif
;

/* G~[g}V̏ */
EXTERN const Machineinfo *machineInfo[MACHINE_LAST + 1]
#if defined(DEF_GLOBAL)
= {
	&e200info,
	&g815info,
	&g850info
}
#endif
;

/* CPU/ */
EXTERN Z80stat z80;	/* CPU */
EXTERN int freqCPU;	/* CPUg */
EXTERN int romBanks;	/* ROMoN */
EXTERN int romBank;	/* ݂ROMoNԍ */
EXTERN int exBank;	/* ݂EXROMoNԍ */
EXTERN int ramBank;	/* ݂RAMoNԍ */
EXTERN uint8 ioReset;	/* Ӌ@pZbgM */
EXTERN uint8 onCd;	/* CDMɂON(PC-G850) */
EXTERN uint8 m1Wait;	/* M1Mwait(PC-G850) */
EXTERN uint8 ioWait;	/* I/O wait(PC-G850) */
EXTERN uint8 csClk;	/* Eᑬ[h(PC-G850) */
EXTERN uint8 memory[0x10001];	/*  */
EXTERN uint8 *rom;	/* ROM */

/* 荞 */
EXTERN uint8 interruptType;	/* 荞ݗv */
EXTERN uint8 interruptMask;	/* 荞݃}XN */

/* ^C} */
EXTERN uint8 timer;	/* ^C} */

/* L[{[h */
EXTERN uint16 keyStrobe;	/* L[Xg[u */
EXTERN uint16 keyStrobeLast;	/* Ōɐݒ肵L[Xg[u */
EXTERN uint8 keyMatrix[10];	/* L[ */
EXTERN uint8 keyShift;	/* VtgL[ */
EXTERN uint8 keyBreak;	/* ONL[ */
EXTERN int keyReset;	/* Zbg{^ */
EXTERN int statesKeyStrobeLast; /* L[Xg[uݒ肵Ƃ̗ݐσXe[g */
EXTERN int statesKeyStrobeClear;	/* L[Xg[uNAXe[g */

/* LCD */
EXTERN uint8 lcdX;	/* AhX */
EXTERN uint8 lcdY;	/* cAhX */
EXTERN uint8 lcdX2;	/* AhX2 (PC-G815) */
EXTERN uint8 lcdY2;	/* cAhX2 (PC-G815) */
EXTERN uint8 lcdBegin;	/* \JnAhX(PC-E200/PC-G815) */
EXTERN uint8 lcdDisabled;	/* LCD OFF(PC-G850) */
EXTERN uint8 lcdTop;	/* \Jnʒu(PC-G850) */
EXTERN uint8 lcdContrast;	/* RgXg(PC-G850) */
EXTERN int lcdEffectMirror;	/* ~[[h(PC-G850) */
EXTERN int lcdEffectBlack;	/* hԂ(PC-G850) */
EXTERN int lcdEffectReverse;	/* ](PC-G850) */
EXTERN int lcdEffectDark;	/* LCDd(PC-G850) */
EXTERN int lcdEffectWhite;	/* hԂ(PC-G850) */
EXTERN uint8 lcdTrim;	/* g(PC-G850) */
EXTERN uint8 vram[166 * 9];	/* VRAM */

/* 11pin I/F */
EXTERN uint8 pin11If;	/* 11pinI/F̓ */
#define PIN11IF_3IO	0	/* 3in 3out I/F */
#define PIN11IF_8PIO	1	/* 8bits PIO    */
#define PIN11IF_UART	2	/* S2dʐM    */
EXTERN uint8 pin11In;	/* 11pin͏ */
EXTERN uint8 xinEnabled;	/* Xin */
EXTERN uint8 io3Out;	/* Xout */
EXTERN uint8 pio8Io;	/* PIOo͕ */
EXTERN uint8 pio8Out;	/* PIOf[^WX^(o) */
EXTERN uint8 uartIo;
#define UARTIO_UASEL	0x01
#define UARTIO_SIOOUT	0x02
#define UARTIO_IROUT	0x04
#define UARTIO_IRST	0x80
EXTERN uint8 uartMode;
#define UARTMODE_BAUD	0x07
#define UARTMODE_INT	0xc0
EXTERN uint8 uartCommand;
#define UARTCOMMAND_TXEN	0x10
#define UARTCOMMAND_RXEN	0x20
#define UARTCOMMAND_ERLL	0x40
#define UARTCOMMAND_SRST	0x80
EXTERN uint8 uartStatus;
#define UARTSTATUS_TXEMP	0x04
#define UARTSTATUS_TXRDY	0x08
#define UARTSTATUS_RXRDY	0x10
#define UARTSTATUS_PERR	0x20
#define UARTSTATUS_OVER	0x40
#define UARTSTATUS_FRME	0x80

/* SIO */
EXTERN char pathSioIn[FILENAME_MAX];	/* SIO̓t@C̃pX */
EXTERN char pathSioOut[FILENAME_MAX];	/* SIOo̓t@C̃pX */
EXTERN int sioMode;	/* SIȌ */
#define SIO_MODE_STOP	0	/* ~ */
#define SIO_MODE_IN	1	/*  */
#define SIO_MODE_OUT	2	/* o */
EXTERN char sioBuffer[0x20000];	/* SIOobt@ */
EXTERN int sioBufferSize;	/* SIOobt@̃TCY */
EXTERN int sioCount;	/* SIOJE^ */
EXTERN int sioBusy;	/* M? */

/* uU[ */
EXTERN int buzzer;	/* uU[[h */
#define BUZZER_NONE	0	/* Ȃ */
#define BUZZER_USE	1	/*  */
#define BUZZER_SYNC	2	/* Ƃ */
EXTERN int soundBufferSize;	/* TEhobt@̃TCY */
EXTERN uint8 *soundReadBuffer;	/* TEhǂݍ݃obt@ */
EXTERN uint8 *soundWriteBuffer;	/* TEh݃obt@ */
EXTERN volatile int soundPlayed;	/* TEho͂? */

/* ݒ */
EXTERN int emulatorMode;	/* G~[^̃[h */
#define EMULATOR_MODE_RUN	0	/* s */
#define EMULATOR_MODE_MENU	1	/* j[\ */
EXTERN int machine;	/* G~[g}V */
EXTERN char dirROM[FILENAME_MAX];	/* ROMC[ŴfBNg */
EXTERN char pathRAM[FILENAME_MAX];	/* RAMۑt@C̃pX */
EXTERN char pathProg[FILENAME_MAX];	/* vÕpX */
EXTERN int useROM;	/* @ROMg? */
EXTERN int freqUpdateIO;	/* I/O̍XV */
EXTERN int beginProg;	/* vO̊JnAhX */
EXTERN int emulateIOCS;	/* IOCSG~[g邩? */
EXTERN int lcdScales;	/* LCD̊K */
EXTERN int noWait;	/* EFCgȂ[h */
EXTERN int zoom;	/* LCD̔{ */
EXTERN uint32 colorBack;	/* LCDwiF */
EXTERN uint32 colorOff;	/* LCD OFFF */
EXTERN uint32 colorOn;	/* LCD ONF */
EXTERN int cellWidth;	/* cell̕ */
EXTERN int cellHeight;	/* cell̍ */
EXTERN int lcdRows;	/* LCD̍s */
EXTERN int lcdCols;	/* LCD̗ */
EXTERN int lcdWidth;	/* LCD̕ */
EXTERN int lcdHeight;	/* LCD̍ */
EXTERN int vramRows;	/* VRAM̍s */
EXTERN int vramCols;	/* VRAM̗ */
EXTERN int vramWidth;	/* VRAM̕ */
EXTERN int vramHeight;	/* VRAM̍ */
EXTERN int closeAsOff;	/* uvOFFL[ƂĈ? */
EXTERN int useJoy;	/* WCXeBbNg? */
EXTERN uint8 joyUp;	/* WCXfBbN̏Ɋ蓖ĂꂽL[ */
EXTERN uint8 joyDown;	/* WCXeBbN̉Ɋ蓖ĂꂽL[ */
EXTERN uint8 joyLeft;	/* WCXeBbN̍Ɋ蓖ĂꂽL[ */
EXTERN uint8 joyRight;	/* WCXeBbN̉EɊ蓖ĂꂽL[ */
EXTERN uint8 joyButton[JOY_BUTTONS];	/* WCXeBbÑ{^Ɋ蓖ĂꂽL[ */
EXTERN int useSoftwareKey;	/* \tgEFAL[g? */
EXTERN int serialTest;	/* VA|[g̃eXgs? */
EXTERN int useFileFilter;	/* t@CtB^g? */

/*
	֐vg^Cv
*/
/* depend.c */
int updateLCD(void);
void updateLCDContrast(void);
void updateLayout(void);
uint8 updateKey(void);
int delay(int);
void updateSerial(void);
int initDepend(void);

/* io.c */
uint8 pin11out(void);

/* iocs.c */
void scrup(void);
void putchr(uint8, uint8, uint8);
int nextRow(uint8 *, uint8 *);
int nextCol(uint8 *, uint8 *);
void prevCol(uint8 *, uint8 *);
int putchrNext(uint8 *, uint8 *, uint8);
uint8 getKeycode(void);
uint8 getKeycodeWait(void);

/* sio.c */
uint8 sioRead(uint8);
int sioLoad(const char *);
uint8 sioWrite(uint8);
int sioSave(const char *);

/* monitor.c */
int loadProg(uint16 *, const char *);
void go(uint16);
void popup(const char *, const char *, ...);
int menu(void);
void monitor(void);

/* sound.c */
int getSoundBufferSize(int);
void writeSound(int);
void flipSoundBuffer(void);

/* util.c */
size_t readBin(const char *, void *, size_t);
size_t writeBin(const char *, const void *, size_t);

/* init.c */
int init(int argc, char *argv[]);

/* g800.c */
int exec(int);

/*
	ˑ
*/
#include "depend.h"

#endif

/*
	Copyright 2005 ~ 2008 maruhiro
	All rights reserved. 

	Redistribution and use in source and binary forms, 
	with or without modification, are permitted provided that 
	the following conditions are met: 

	 1. Redistributions of source code must retain the above copyright notice, 
	    this list of conditions and the following disclaimer. 

	 2. Redistributions in binary form must reproduce the above copyright notice, 
	    this list of conditions and the following disclaimer in the documentation 
	    and/or other materials provided with the distribution. 

	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
	FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
	THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* eof */
