/*
	SHARP PC-G800 series emulator
	
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "g800.h"

/* Yes/No */
const static OptTable tableYesNo[] = {
	{ "y"  , TRUE  },
	{ "yes", TRUE  },
	{ "1"  , TRUE  },
	{ "n"  , FALSE },
	{ "no" , FALSE },
	{ "0"  , FALSE },
	{ NULL, 0 }
};

/* }V */
const static OptTable tableMachine[] = {
	{ "e200", MACHINE_E200 },
	{ "g815", MACHINE_G815 },
	{ "g850", MACHINE_G850 },
	{ NULL, 0 }
};

/* uU[ */
const static OptTable tableBuzzer[] = {
	{ "n",    BUZZER_NONE },
	{ "no",   BUZZER_NONE },
	{ "0",    BUZZER_NONE },
	{ "y",    BUZZER_USE },
	{ "yes",  BUZZER_USE },
	{ "1",    BUZZER_USE },
	{ "sync", BUZZER_SYNC },
	{ "2",    BUZZER_SYNC },
	{ NULL, 0 }
};

/* G~[g鑤̃L[ */
const static OptTable tableKey[] = {
	{ "none"     , 0                },
	{ "backspace", KEY_BACKSPACE    },
	{ "tab"      , KEY_TAB          },
	{ "clear"    , KEY_CLEAR        },
	{ "return"   , KEY_RETURN       },
	{ "enter"    , KEY_RETURN       },
	{ "pause"    , KEY_PAUSE        },
	{ "escape"   , KEY_ESCAPE       },
	{ "space"    , KEY_SPACE        },
	{ "!"        , KEY_EXCLAIM      },
	{ "\""       , KEY_QUOTEDBL     },
	{ "hash"     , KEY_HASH         },
	{ "sharp"    , KEY_HASH         },
	{ "$"        , KEY_DOLLAR       },
	{ "&"        , KEY_AMPERSAND    },
	{ "+"        , KEY_PLUS         },
	{ ","        , KEY_COMMA        },
	{ "-"        , KEY_MINUS        },
	{ "."        , KEY_PERIOD       },
	{ "/"        , KEY_SLASH        },
	{ "0"        , KEY_0            },
	{ "1"        , KEY_1            },
	{ "2"        , KEY_2            },
	{ "3"        , KEY_3            },
	{ "4"        , KEY_4            },
	{ "5"        , KEY_5            },
	{ "6"        , KEY_6            },
	{ "7"        , KEY_7            },
	{ "8"        , KEY_8            },
	{ "9"        , KEY_9            },
	{ ":"        , KEY_COLON        },
	{ ";"        , KEY_SEMICOLON    },
	{ "="        , KEY_EQUALS       },
	{ ">"        , KEY_GREATER      },
	{ "?"        , KEY_QUESTION     },
	{ "["        , KEY_LEFTBRACKET  },
	{ "\\"       , KEY_BACKSLASH    },
	{ "]"        , KEY_RIGHTBRACKET },
	{ "^"        , KEY_CARET        },
	{ "_"        , KEY_UNDERSCORE   },
	{ "`"        , KEY_BACKQUOTE    },
	{ "@"        , KEY_AT           },
	{ "a"        , KEY_A            },
	{ "b"        , KEY_B            },
	{ "c"        , KEY_C            },
	{ "d"        , KEY_D            },
	{ "e"        , KEY_E            },
	{ "f"        , KEY_F            },
	{ "g"        , KEY_G            },
	{ "h"        , KEY_H            },
	{ "i"        , KEY_I            },
	{ "j"        , KEY_J            },
	{ "k"        , KEY_K            },
	{ "l"        , KEY_L            },
	{ "m"        , KEY_M            },
	{ "n"        , KEY_N            },
	{ "o"        , KEY_O            },
	{ "p"        , KEY_P            },
	{ "q"        , KEY_Q            },
	{ "r"        , KEY_R            },
	{ "s"        , KEY_S            },
	{ "t"        , KEY_T            },
	{ "u"        , KEY_U            },
	{ "v"        , KEY_V            },
	{ "w"        , KEY_W            },
	{ "x"        , KEY_X            },
	{ "y"        , KEY_Y            },
	{ "z"        , KEY_Z            },
	{ "delete"   , KEY_DELETE       },
	{ "n0"       , KEY_KP0          },
	{ "n1"       , KEY_KP1          },
	{ "n2"       , KEY_KP2          },
	{ "n3"       , KEY_KP3          },
	{ "n4"       , KEY_KP4          },
	{ "n5"       , KEY_KP5          },
	{ "n6"       , KEY_KP6          },
	{ "n7"       , KEY_KP7          },
	{ "n8"       , KEY_KP8          },
	{ "n9"       , KEY_KP9          },
	{ "n."       , KEY_KP_PERIOD    },
	{ "n/"       , KEY_KP_DIVIDE    },
	{ "n*"       , KEY_KP_MULTIPLY  },
	{ "n-"       , KEY_KP_MINUS     },
	{ "n+"       , KEY_KP_PLUS      },
	{ "nreturn"  , KEY_KP_ENTER     },
	{ "nenter"   , KEY_KP_ENTER     },
	{ "n="       , KEY_KP_EQUALS    },
	{ "f1"       , KEY_F1           },
	{ "f2"       , KEY_F2           },
	{ "f3"       , KEY_F3           },
	{ "f4"       , KEY_F4           },
	{ "f5"       , KEY_F5           },
	{ "f6"       , KEY_F6           },
	{ "f7"       , KEY_F7           },
	{ "f8"       , KEY_F8           },
	{ "f9"       , KEY_F9           },
	{ "f10"      , KEY_F10          },
	{ "f11"      , KEY_F11          },
	{ "f12"      , KEY_F12          },
	{ "f13"      , KEY_F13          },
	{ "f14"      , KEY_F14          },
	{ "f15"      , KEY_F15          },
	{ "up"       , KEY_UP           },
	{ "down"     , KEY_DOWN         },
	{ "right"    , KEY_RIGHT        },
	{ "left"     , KEY_LEFT         },
	{ "insert"   , KEY_INSERT       },
	{ "home"     , KEY_HOME         },
	{ "end"      , KEY_END          },
	{ "pageup"   , KEY_PAGEUP       },
	{ "pagedown" , KEY_PAGEDOWN     },
	{ "numlock"  , KEY_NUMLOCK      },
	{ "capslock" , KEY_CAPSLOCK     },
	{ "scrollock", KEY_SCROLLOCK    },
	{ "rshift"   , KEY_RSHIFT       },
	{ "lshift"   , KEY_LSHIFT       },
	{ "rctrl"    , KEY_RCTRL        },
	{ "lctrl"    , KEY_LCTRL        },
	{ "ralt"     , KEY_RALT         },
	{ "lalt"     , KEY_LALT         },
	{ "lsuper"   , KEY_LSUPER       },
	{ "rsuper"   , KEY_RSUPER       },
	{ "mode"     , KEY_MODE         },
	{ "compose"  , KEY_COMPOSE      },
	{ "help"     , KEY_HELP         },
	{ "print"    , KEY_PRINT        },
	{ "sysreq"   , KEY_SYSREQ       },
	{ "break"    , KEY_BREAK        },
	{ "menu"     , KEY_MENU         },
	{ "power"    , KEY_POWER        },
	{ "euro"     , KEY_EURO         },
	/*{ "undo"     , KEY_UNDO         },*/
	{ NULL, 0 }
};

/* PC-G800̃L[ */
const static OptTable tableGkey[] = {
	{ "off"      , GKEY_OFF       },
	{ "q"        , GKEY_Q         },
	{ "w"        , GKEY_W         },
	{ "e"        , GKEY_E         },
	{ "r"        , GKEY_R         },
	{ "t"        , GKEY_T         },
	{ "y"        , GKEY_Y         },
	{ "u"        , GKEY_U         },
	{ "a"        , GKEY_A         },
	{ "s"        , GKEY_S         },
	{ "d"        , GKEY_D         },
	{ "f"        , GKEY_F         },
	{ "g"        , GKEY_G         },
	{ "h"        , GKEY_H         },
	{ "j"        , GKEY_J         },
	{ "k"        , GKEY_K         },
	{ "z"        , GKEY_Z         },
	{ "x"        , GKEY_X         },
	{ "c"        , GKEY_C         },
	{ "v"        , GKEY_V         },
	{ "b"        , GKEY_B         },
	{ "n"        , GKEY_N         },
	{ "m"        , GKEY_M         },
	{ ","        , GKEY_COMMA     },
	{ "basic"    , GKEY_BASIC     },
	{ "text"     , GKEY_TEXT      },
	{ "capslock" , GKEY_CAPS      },
	{ "kana"     , GKEY_KANA      },
	{ "tab"      , GKEY_TAB       },
	{ "space"    , GKEY_SPACE     },
	{ "down"     , GKEY_DOWN      },
	{ "up"       , GKEY_UP        },
	{ "left"     , GKEY_LEFT      },
	{ "right"    , GKEY_RIGHT     },
	{ "ans"      , GKEY_ANS       },
	{ "0"        , GKEY_0         },
	{ "."        , GKEY_PERIOD    },
	{ "="        , GKEY_EQUAL     },
	{ "+"        , GKEY_PLUS      },
	{ "return"   , GKEY_RETURN    },
	{ "l"        , GKEY_L         },
	{ ";"        , GKEY_SEMICOLON },
	{ "const"    , GKEY_CONST     },
	{ "1"        , GKEY_1         },
	{ "2"        , GKEY_2         },
	{ "3"        , GKEY_3         },
	{ "-"        , GKEY_MINUS     },
	{ "m+"       , GKEY_MPLUS     },
	{ "i"        , GKEY_I         },
	{ "o"        , GKEY_O         },
	{ "insert"   , GKEY_INSERT    },
	{ "4"        , GKEY_4         },
	{ "5"        , GKEY_5         },
	{ "6"        , GKEY_6         },
	{ "*"        , GKEY_ASTER     },
	{ "rcm"      , GKEY_RCM       },
	{ "p"        , GKEY_P         },
	{ "backspace", GKEY_BACKSPACE },
	{ "pi"       , GKEY_PI        },
	{ "7"        , GKEY_7         },
	{ "8"        , GKEY_8         },
	{ "9"        , GKEY_9         },
	{ "/"        , GKEY_SLASH     },
	{ "("        , GKEY_LKAKKO    },
	{ "npr"      , GKEY_NPR       },
	{ "deg"      , GKEY_DEG       },
	{ "sqr"      , GKEY_SQR       },
	{ "squ"      , GKEY_SQU       },
	{ "^"        , GKEY_HAT       },
	{ ")"        , GKEY_RKAKKO    },
	{ "rcp"      , GKEY_RCP       },
	{ "mdf"      , GKEY_MDF       },
	{ "2ndf"     , GKEY_2NDF      },
	{ "sin"      , GKEY_SIN       },
	{ "cos"      , GKEY_COS       },
	{ "ln"       , GKEY_LN        },
	{ "log"      , GKEY_LOG       },
	{ "tan"      , GKEY_TAN       },
	{ "fe"       , GKEY_FE        },
	{ "cls"      , GKEY_CLS       },
	{ "on"       , GKEY_BREAK     },
	{ "shift"    , GKEY_SHIFT     },
	{ "reset"    , GKEY_RESET     },
	{ "trace"    , GKEY_DEBUG     },
	{ "11pin1"   , GKEY_11PIN1    },
	{ "11pin2"   , GKEY_11PIN2    },
	{ "11pin3"   , GKEY_11PIN3    },
	{ "11pin4"   , GKEY_11PIN4    },
	{ "11pin5"   , GKEY_11PIN5    },
	{ "11pin6"   , GKEY_11PIN6    },
	{ "11pin7"   , GKEY_11PIN7    },
	{ "11pin8"   , GKEY_11PIN8    },
	{ "menu"     , GKEY_MENU      },
	{ NULL, 0 }
};

/*
	
*/
int init(int argc, char *argv[])
{
	Conf conf[256];
	const Machineinfo *m;
	const OptTable *p;
	int i, trace, jiskey, success;

	/* Ǎt@CƊJnAhX */
	beginProg = 0;
	for(i = 1; i < argc; i++)
		if(*argv[i] != '-') {
			strcpy(pathProg, argv[i]);
			for(i = i + 1; i < argc; i++)
				if(*argv[i] != '-')
					sscanf(argv[i], "%x", &beginProg);
		}

	success = (getConfig(conf, sizeof(conf) / sizeof(conf[0]), "g800config", argc, argv) != NULL);

	/* ݒ */
	trace = optTable(conf, "debug", tableYesNo, FALSE);
	if((freqUpdateIO = optInt(conf, "refresh", 60)) > 360)
		freqUpdateIO = 360;
	setHomeDir(pathSioIn, optText(conf, "sio_in", ""));
	setHomeDir(pathSioOut, optText(conf, "sio_out", ""));
	noWait = optTable(conf, "no_wait", tableYesNo, FALSE);
	serialTest = optTable(conf, "serial_test", tableYesNo, FALSE);
	jiskey = optTable(conf, "jiskey", tableYesNo, FALSE);
	useSoftwareKey = optTable(conf, "software_key", tableYesNo, FALSE);

	/*  */
	setHomeDir(pathRAM, optText(conf, "ram_path", ""));
	setHomeDir(dirROM, optText(conf, "rom_dir", ""));
	useROM = (strcmp(dirROM, "") != 0);

	/* G~[V */
	machine = optTable(conf, "machine", tableMachine, MACHINE_G850);
	emulateIOCS = optTable(conf, "emulate_iocs", tableYesNo, !useROM);
	if((lcdScales = optInt(conf, "lcd_scales", 2)) == 1)
		lcdScales = 2;
	freqCPU = optInt(conf, "clock", 0) * 1000;

	/* LCD */
	zoom      = optInt(conf, "zoom",     3);
	colorBack = optHex(conf, "lcd_back", 0xcceebb);
	colorOff  = optHex(conf, "lcd_off",  0xaaddbb);
	colorOn   = optHex(conf, "lcd_on",   0x002211);

	/* L[{[h */
	for(p = tableGkey; p->string != NULL; p++)
		keyconv[optTable(conf, p->string, tableKey, KEY_NONE)] = p->value;
	closeAsOff = optTable(conf, "close_as_off", tableYesNo, FALSE);

	/* WCXeBbN */
	useJoy = optTable(conf, "use_joy", tableYesNo, TRUE);
	joyUp         = optTable(conf, "joy_up",       tableGkey, GKEY_UP    );
	joyDown       = optTable(conf, "joy_down",     tableGkey, GKEY_DOWN  );
	joyLeft       = optTable(conf, "joy_left",     tableGkey, GKEY_LEFT  );
	joyRight      = optTable(conf, "joy_right",    tableGkey, GKEY_RIGHT );
	joyButton[0]  = optTable(conf, "joy_button1",  tableGkey, GKEY_SPACE );
	joyButton[1]  = optTable(conf, "joy_button2",  tableGkey, GKEY_RETURN);
	joyButton[2]  = optTable(conf, "joy_button3",  tableGkey, GKEY_NONE  );
	joyButton[3]  = optTable(conf, "joy_button4",  tableGkey, GKEY_NONE  );
	joyButton[4]  = optTable(conf, "joy_button5",  tableGkey, GKEY_NONE  );
	joyButton[5]  = optTable(conf, "joy_button6",  tableGkey, GKEY_NONE  );
	joyButton[6]  = optTable(conf, "joy_button7",  tableGkey, GKEY_NONE  );
	joyButton[7]  = optTable(conf, "joy_button8",  tableGkey, GKEY_NONE  );
	joyButton[8]  = optTable(conf, "joy_button9",  tableGkey, GKEY_NONE  );
	joyButton[9]  = optTable(conf, "joy_button10", tableGkey, GKEY_NONE  );
	joyButton[10] = optTable(conf, "joy_button11", tableGkey, GKEY_NONE  );
	joyButton[11] = optTable(conf, "joy_button12", tableGkey, GKEY_NONE  );

	/* LCD */
	if(zoom < 0)
		zoom = 2;
	m = machineInfo[machine];
	cellWidth  = m->cell_width;
	cellHeight = m->cell_height;
	lcdCols    = m->lcd_cols;
	lcdRows    = m->lcd_rows;
	vramCols   = m->vram_cols;
	vramRows   = m->vram_rows;
	lcdWidth   = cellWidth * lcdCols;
	lcdHeight  = 8 * lcdRows;
	vramWidth  = m->vram_width;
	vramHeight = m->vram_height;
	statesKeyStrobeClear = m->states_keystrobe_clear;
	freqCPU    = (freqCPU != 0 ? freqCPU: m->cpu_clocks);

	/* VAo */
	if(strcmp(pathSioOut, "") != 0)
		sioSave(pathSioOut);
	if(strcmp(pathSioIn, "") != 0)
		sioLoad(pathSioIn);

	/* uU[ */
	buzzer = optTable(conf, "buzzer", tableBuzzer, TRUE);
	if(buzzer != BUZZER_NONE) {
		soundBufferSize = getSoundBufferSize(freqUpdateIO);
		freqUpdateIO    = FREQ_SOUND / soundBufferSize;
		soundReadBuffer  = calloc(1, soundBufferSize * 2);
		soundWriteBuffer = soundReadBuffer + soundBufferSize;
	}

	/* win32SDL̖(?)邽߃L[ւ */
	if(jiskey) {
		keyconv[KEY_BACKSLASH]    = keyconv[KEY_RIGHTBRACKET];
		keyconv[KEY_RIGHTBRACKET] = keyconv[KEY_LEFTBRACKET];
		keyconv[KEY_EQUALS]       = keyconv[KEY_CARET];
		keyconv[KEY_QUOTE]        = keyconv[KEY_COLON];
		keyconv[KEY_LEFTBRACKET]  = keyconv[KEY_AT];
	}

	/* t@CtB^g? */
	useFileFilter = optTable(conf, "file_filter", tableYesNo, FALSE);

	initDepend();
	z80.m = memory;
	z80.i.trace = trace;
	z80.i.emulate_subroutine = emulateIOCS;
	lcdContrast = (machine == MACHINE_G850 ? 0: 0x0f);
	updateLCDContrast();

	return success;
}

/*
	Copyright 2005 ~ 2008 maruhiro
	All rights reserved. 

	Redistribution and use in source and binary forms, 
	with or without modification, are permitted provided that 
	the following conditions are met: 

	 1. Redistributions of source code must retain the above copyright notice, 
	    this list of conditions and the following disclaimer. 

	 2. Redistributions in binary form must reproduce the above copyright notice, 
	    this list of conditions and the following disclaimer in the documentation 
	    and/or other materials provided with the distribution. 

	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
	FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
	THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
	OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
	ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* eof */
